import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSEvent(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSLeftMouseDown, 1)
        self.assertEqual(AppKit.NSLeftMouseUp, 2)
        self.assertEqual(AppKit.NSRightMouseDown, 3)
        self.assertEqual(AppKit.NSRightMouseUp, 4)
        self.assertEqual(AppKit.NSMouseMoved, 5)
        self.assertEqual(AppKit.NSLeftMouseDragged, 6)
        self.assertEqual(AppKit.NSRightMouseDragged, 7)
        self.assertEqual(AppKit.NSMouseEntered, 8)
        self.assertEqual(AppKit.NSMouseExited, 9)
        self.assertEqual(AppKit.NSKeyDown, 10)
        self.assertEqual(AppKit.NSKeyUp, 11)
        self.assertEqual(AppKit.NSFlagsChanged, 12)
        self.assertEqual(AppKit.NSAppKitDefined, 13)
        self.assertEqual(AppKit.NSSystemDefined, 14)
        self.assertEqual(AppKit.NSApplicationDefined, 15)
        self.assertEqual(AppKit.NSPeriodic, 16)
        self.assertEqual(AppKit.NSCursorUpdate, 17)
        self.assertEqual(AppKit.NSScrollWheel, 22)
        self.assertEqual(AppKit.NSTabletPoint, 23)
        self.assertEqual(AppKit.NSTabletProximity, 24)
        self.assertEqual(AppKit.NSOtherMouseDown, 25)
        self.assertEqual(AppKit.NSOtherMouseUp, 26)
        self.assertEqual(AppKit.NSOtherMouseDragged, 27)

        self.assertEqual(AppKit.NSEventTypeLeftMouseDown, 1)
        self.assertEqual(AppKit.NSEventTypeLeftMouseUp, 2)
        self.assertEqual(AppKit.NSEventTypeRightMouseDown, 3)
        self.assertEqual(AppKit.NSEventTypeRightMouseUp, 4)
        self.assertEqual(AppKit.NSEventTypeMouseMoved, 5)
        self.assertEqual(AppKit.NSEventTypeLeftMouseDragged, 6)
        self.assertEqual(AppKit.NSEventTypeRightMouseDragged, 7)
        self.assertEqual(AppKit.NSEventTypeMouseEntered, 8)
        self.assertEqual(AppKit.NSEventTypeMouseExited, 9)
        self.assertEqual(AppKit.NSEventTypeKeyDown, 10)
        self.assertEqual(AppKit.NSEventTypeKeyUp, 11)
        self.assertEqual(AppKit.NSEventTypeFlagsChanged, 12)
        self.assertEqual(AppKit.NSEventTypeAppKitDefined, 13)
        self.assertEqual(AppKit.NSEventTypeSystemDefined, 14)
        self.assertEqual(AppKit.NSEventTypeApplicationDefined, 15)
        self.assertEqual(AppKit.NSEventTypePeriodic, 16)
        self.assertEqual(AppKit.NSEventTypeCursorUpdate, 17)
        self.assertEqual(AppKit.NSEventTypeScrollWheel, 22)
        self.assertEqual(AppKit.NSEventTypeTabletPoint, 23)
        self.assertEqual(AppKit.NSEventTypeTabletProximity, 24)
        self.assertEqual(AppKit.NSEventTypeOtherMouseDown, 25)
        self.assertEqual(AppKit.NSEventTypeOtherMouseUp, 26)
        self.assertEqual(AppKit.NSEventTypeOtherMouseDragged, 27)

        self.assertEqual(AppKit.NSLeftMouseDownMask, 1 << AppKit.NSLeftMouseDown)
        self.assertEqual(AppKit.NSLeftMouseUpMask, 1 << AppKit.NSLeftMouseUp)
        self.assertEqual(AppKit.NSRightMouseDownMask, 1 << AppKit.NSRightMouseDown)
        self.assertEqual(AppKit.NSRightMouseUpMask, 1 << AppKit.NSRightMouseUp)
        self.assertEqual(AppKit.NSMouseMovedMask, 1 << AppKit.NSMouseMoved)
        self.assertEqual(AppKit.NSLeftMouseDraggedMask, 1 << AppKit.NSLeftMouseDragged)
        self.assertEqual(
            AppKit.NSRightMouseDraggedMask, 1 << AppKit.NSRightMouseDragged
        )
        self.assertEqual(AppKit.NSMouseEnteredMask, 1 << AppKit.NSMouseEntered)
        self.assertEqual(AppKit.NSMouseExitedMask, 1 << AppKit.NSMouseExited)
        self.assertEqual(AppKit.NSKeyDownMask, 1 << AppKit.NSKeyDown)
        self.assertEqual(AppKit.NSKeyUpMask, 1 << AppKit.NSKeyUp)
        self.assertEqual(AppKit.NSFlagsChangedMask, 1 << AppKit.NSFlagsChanged)
        self.assertEqual(AppKit.NSAppKitDefinedMask, 1 << AppKit.NSAppKitDefined)
        self.assertEqual(AppKit.NSSystemDefinedMask, 1 << AppKit.NSSystemDefined)
        self.assertEqual(
            AppKit.NSApplicationDefinedMask, 1 << AppKit.NSApplicationDefined
        )
        self.assertEqual(AppKit.NSPeriodicMask, 1 << AppKit.NSPeriodic)
        self.assertEqual(AppKit.NSCursorUpdateMask, 1 << AppKit.NSCursorUpdate)
        self.assertEqual(AppKit.NSScrollWheelMask, 1 << AppKit.NSScrollWheel)
        self.assertEqual(AppKit.NSTabletPointMask, 1 << AppKit.NSTabletPoint)
        self.assertEqual(AppKit.NSTabletProximityMask, 1 << AppKit.NSTabletProximity)
        self.assertEqual(AppKit.NSOtherMouseDownMask, 1 << AppKit.NSOtherMouseDown)
        self.assertEqual(AppKit.NSOtherMouseUpMask, 1 << AppKit.NSOtherMouseUp)
        self.assertEqual(
            AppKit.NSOtherMouseDraggedMask, 1 << AppKit.NSOtherMouseDragged
        )
        self.assertEqual(AppKit.NSAnyEventMask, AppKit.NSUIntegerMax)

        self.assertEqual(
            AppKit.NSEventMaskLeftMouseDown, 1 << AppKit.NSEventTypeLeftMouseDown
        )
        self.assertEqual(
            AppKit.NSEventMaskLeftMouseUp, 1 << AppKit.NSEventTypeLeftMouseUp
        )
        self.assertEqual(
            AppKit.NSEventMaskRightMouseDown, 1 << AppKit.NSEventTypeRightMouseDown
        )
        self.assertEqual(
            AppKit.NSEventMaskRightMouseUp, 1 << AppKit.NSEventTypeRightMouseUp
        )
        self.assertEqual(
            AppKit.NSEventMaskMouseMoved, 1 << AppKit.NSEventTypeMouseMoved
        )
        self.assertEqual(
            AppKit.NSEventMaskLeftMouseDragged, 1 << AppKit.NSEventTypeLeftMouseDragged
        )
        self.assertEqual(
            AppKit.NSEventMaskRightMouseDragged,
            1 << AppKit.NSEventTypeRightMouseDragged,
        )
        self.assertEqual(
            AppKit.NSEventMaskMouseEntered, 1 << AppKit.NSEventTypeMouseEntered
        )
        self.assertEqual(
            AppKit.NSEventMaskMouseExited, 1 << AppKit.NSEventTypeMouseExited
        )
        self.assertEqual(AppKit.NSEventMaskKeyDown, 1 << AppKit.NSEventTypeKeyDown)
        self.assertEqual(AppKit.NSEventMaskKeyUp, 1 << AppKit.NSEventTypeKeyUp)
        self.assertEqual(
            AppKit.NSEventMaskFlagsChanged, 1 << AppKit.NSEventTypeFlagsChanged
        )
        self.assertEqual(
            AppKit.NSEventMaskAppKitDefined, 1 << AppKit.NSEventTypeAppKitDefined
        )
        self.assertEqual(
            AppKit.NSEventMaskSystemDefined, 1 << AppKit.NSEventTypeSystemDefined
        )
        self.assertEqual(
            AppKit.NSEventMaskApplicationDefined,
            1 << AppKit.NSEventTypeApplicationDefined,
        )
        self.assertEqual(AppKit.NSEventMaskPeriodic, 1 << AppKit.NSEventTypePeriodic)
        self.assertEqual(
            AppKit.NSEventMaskCursorUpdate, 1 << AppKit.NSEventTypeCursorUpdate
        )
        self.assertEqual(
            AppKit.NSEventMaskScrollWheel, 1 << AppKit.NSEventTypeScrollWheel
        )
        self.assertEqual(
            AppKit.NSEventMaskTabletPoint, 1 << AppKit.NSEventTypeTabletPoint
        )
        self.assertEqual(
            AppKit.NSEventMaskTabletProximity, 1 << AppKit.NSEventTypeTabletProximity
        )
        self.assertEqual(
            AppKit.NSEventMaskOtherMouseDown, 1 << AppKit.NSEventTypeOtherMouseDown
        )
        self.assertEqual(
            AppKit.NSEventMaskOtherMouseUp, 1 << AppKit.NSEventTypeOtherMouseUp
        )
        self.assertEqual(
            AppKit.NSEventMaskOtherMouseDragged,
            1 << AppKit.NSEventTypeOtherMouseDragged,
        )

        self.assertEqual(AppKit.NSAlphaShiftKeyMask, 1 << 16)
        self.assertEqual(AppKit.NSShiftKeyMask, 1 << 17)
        self.assertEqual(AppKit.NSControlKeyMask, 1 << 18)
        self.assertEqual(AppKit.NSAlternateKeyMask, 1 << 19)
        self.assertEqual(AppKit.NSCommandKeyMask, 1 << 20)
        self.assertEqual(AppKit.NSNumericPadKeyMask, 1 << 21)
        self.assertEqual(AppKit.NSHelpKeyMask, 1 << 22)
        self.assertEqual(AppKit.NSFunctionKeyMask, 1 << 23)
        self.assertEqual(AppKit.NSDeviceIndependentModifierFlagsMask, 0xFFFF0000)

        self.assertEqual(AppKit.NSEventModifierFlagCapsLock, 1 << 16)
        self.assertEqual(AppKit.NSEventModifierFlagShift, 1 << 17)
        self.assertEqual(AppKit.NSEventModifierFlagControl, 1 << 18)
        self.assertEqual(AppKit.NSEventModifierFlagOption, 1 << 19)
        self.assertEqual(AppKit.NSEventModifierFlagCommand, 1 << 20)
        self.assertEqual(AppKit.NSEventModifierFlagNumericPad, 1 << 21)
        self.assertEqual(AppKit.NSEventModifierFlagHelp, 1 << 22)
        self.assertEqual(AppKit.NSEventModifierFlagFunction, 1 << 23)
        self.assertEqual(
            AppKit.NSEventModifierFlagDeviceIndependentFlagsMask, 0xFFFF0000
        )

        self.assertEqual(AppKit.NSUnknownPointingDevice, 0)
        self.assertEqual(AppKit.NSPenPointingDevice, 1)
        self.assertEqual(AppKit.NSCursorPointingDevice, 2)
        self.assertEqual(AppKit.NSEraserPointingDevice, 3)

        self.assertEqual(AppKit.NSPointingDeviceTypeUnknown, 0)
        self.assertEqual(AppKit.NSPointingDeviceTypePen, 1)
        self.assertEqual(AppKit.NSPointingDeviceTypeCursor, 2)
        self.assertEqual(AppKit.NSPointingDeviceTypeEraser, 3)

        self.assertEqual(AppKit.NSPenTipMask, 1)
        self.assertEqual(AppKit.NSPenLowerSideMask, 2)
        self.assertEqual(AppKit.NSPenUpperSideMask, 4)

        self.assertEqual(AppKit.NSEventButtonMaskPenTip, 1)
        self.assertEqual(AppKit.NSEventButtonMaskPenLowerSide, 2)
        self.assertEqual(AppKit.NSEventButtonMaskPenUpperSide, 4)

        self.assertEqual(AppKit.NSWindowExposedEventType, 0)
        self.assertEqual(AppKit.NSApplicationActivatedEventType, 1)
        self.assertEqual(AppKit.NSApplicationDeactivatedEventType, 2)
        self.assertEqual(AppKit.NSWindowMovedEventType, 4)
        self.assertEqual(AppKit.NSScreenChangedEventType, 8)
        self.assertEqual(AppKit.NSAWTEventType, 16)
        self.assertEqual(AppKit.NSPowerOffEventType, 1)

        self.assertEqual(AppKit.NSMouseEventSubtype, 0)
        self.assertEqual(AppKit.NSTabletPointEventSubtype, 1)
        self.assertEqual(AppKit.NSTabletProximityEventSubtype, 2)
        self.assertEqual(AppKit.NSTouchEventSubtype, 3)

        self.assertEqual(AppKit.NSEventSubtypeMouseEvent, 0)
        self.assertEqual(AppKit.NSEventSubtypeTabletPoint, 1)
        self.assertEqual(AppKit.NSEventSubtypeTabletProximity, 2)
        self.assertEqual(AppKit.NSEventSubtypeTouch, 3)

        self.assertEqual(AppKit.NSEventSubtypeWindowExposed, 0)
        self.assertEqual(AppKit.NSEventSubtypeApplicationActivated, 1)
        self.assertEqual(AppKit.NSEventSubtypeApplicationDeactivated, 2)
        self.assertEqual(AppKit.NSEventSubtypeWindowMoved, 4)
        self.assertEqual(AppKit.NSEventSubtypeScreenChanged, 8)
        self.assertEqual(AppKit.NSEventSubtypePowerOff, 1)

        self.assertEqual(AppKit.NSUpArrowFunctionKey, chr(0xF700))
        self.assertEqual(AppKit.NSDownArrowFunctionKey, chr(0xF701))
        self.assertEqual(AppKit.NSLeftArrowFunctionKey, chr(0xF702))
        self.assertEqual(AppKit.NSRightArrowFunctionKey, chr(0xF703))
        self.assertEqual(AppKit.NSF1FunctionKey, chr(0xF704))
        self.assertEqual(AppKit.NSF2FunctionKey, chr(0xF705))
        self.assertEqual(AppKit.NSF3FunctionKey, chr(0xF706))
        self.assertEqual(AppKit.NSF4FunctionKey, chr(0xF707))
        self.assertEqual(AppKit.NSF5FunctionKey, chr(0xF708))
        self.assertEqual(AppKit.NSF6FunctionKey, chr(0xF709))
        self.assertEqual(AppKit.NSF7FunctionKey, chr(0xF70A))
        self.assertEqual(AppKit.NSF8FunctionKey, chr(0xF70B))
        self.assertEqual(AppKit.NSF9FunctionKey, chr(0xF70C))
        self.assertEqual(AppKit.NSF10FunctionKey, chr(0xF70D))
        self.assertEqual(AppKit.NSF11FunctionKey, chr(0xF70E))
        self.assertEqual(AppKit.NSF12FunctionKey, chr(0xF70F))
        self.assertEqual(AppKit.NSF13FunctionKey, chr(0xF710))
        self.assertEqual(AppKit.NSF14FunctionKey, chr(0xF711))
        self.assertEqual(AppKit.NSF15FunctionKey, chr(0xF712))
        self.assertEqual(AppKit.NSF16FunctionKey, chr(0xF713))
        self.assertEqual(AppKit.NSF17FunctionKey, chr(0xF714))
        self.assertEqual(AppKit.NSF18FunctionKey, chr(0xF715))
        self.assertEqual(AppKit.NSF19FunctionKey, chr(0xF716))
        self.assertEqual(AppKit.NSF20FunctionKey, chr(0xF717))
        self.assertEqual(AppKit.NSF21FunctionKey, chr(0xF718))
        self.assertEqual(AppKit.NSF22FunctionKey, chr(0xF719))
        self.assertEqual(AppKit.NSF23FunctionKey, chr(0xF71A))
        self.assertEqual(AppKit.NSF24FunctionKey, chr(0xF71B))
        self.assertEqual(AppKit.NSF25FunctionKey, chr(0xF71C))
        self.assertEqual(AppKit.NSF26FunctionKey, chr(0xF71D))
        self.assertEqual(AppKit.NSF27FunctionKey, chr(0xF71E))
        self.assertEqual(AppKit.NSF28FunctionKey, chr(0xF71F))
        self.assertEqual(AppKit.NSF29FunctionKey, chr(0xF720))
        self.assertEqual(AppKit.NSF30FunctionKey, chr(0xF721))
        self.assertEqual(AppKit.NSF31FunctionKey, chr(0xF722))
        self.assertEqual(AppKit.NSF32FunctionKey, chr(0xF723))
        self.assertEqual(AppKit.NSF33FunctionKey, chr(0xF724))
        self.assertEqual(AppKit.NSF34FunctionKey, chr(0xF725))
        self.assertEqual(AppKit.NSF35FunctionKey, chr(0xF726))
        self.assertEqual(AppKit.NSInsertFunctionKey, chr(0xF727))
        self.assertEqual(AppKit.NSDeleteFunctionKey, chr(0xF728))
        self.assertEqual(AppKit.NSHomeFunctionKey, chr(0xF729))
        self.assertEqual(AppKit.NSBeginFunctionKey, chr(0xF72A))
        self.assertEqual(AppKit.NSEndFunctionKey, chr(0xF72B))
        self.assertEqual(AppKit.NSPageUpFunctionKey, chr(0xF72C))
        self.assertEqual(AppKit.NSPageDownFunctionKey, chr(0xF72D))
        self.assertEqual(AppKit.NSPrintScreenFunctionKey, chr(0xF72E))
        self.assertEqual(AppKit.NSScrollLockFunctionKey, chr(0xF72F))
        self.assertEqual(AppKit.NSPauseFunctionKey, chr(0xF730))
        self.assertEqual(AppKit.NSSysReqFunctionKey, chr(0xF731))
        self.assertEqual(AppKit.NSBreakFunctionKey, chr(0xF732))
        self.assertEqual(AppKit.NSResetFunctionKey, chr(0xF733))
        self.assertEqual(AppKit.NSStopFunctionKey, chr(0xF734))
        self.assertEqual(AppKit.NSMenuFunctionKey, chr(0xF735))
        self.assertEqual(AppKit.NSUserFunctionKey, chr(0xF736))
        self.assertEqual(AppKit.NSSystemFunctionKey, chr(0xF737))
        self.assertEqual(AppKit.NSPrintFunctionKey, chr(0xF738))
        self.assertEqual(AppKit.NSClearLineFunctionKey, chr(0xF739))
        self.assertEqual(AppKit.NSClearDisplayFunctionKey, chr(0xF73A))
        self.assertEqual(AppKit.NSInsertLineFunctionKey, chr(0xF73B))
        self.assertEqual(AppKit.NSDeleteLineFunctionKey, chr(0xF73C))
        self.assertEqual(AppKit.NSInsertCharFunctionKey, chr(0xF73D))
        self.assertEqual(AppKit.NSDeleteCharFunctionKey, chr(0xF73E))
        self.assertEqual(AppKit.NSPrevFunctionKey, chr(0xF73F))
        self.assertEqual(AppKit.NSNextFunctionKey, chr(0xF740))
        self.assertEqual(AppKit.NSSelectFunctionKey, chr(0xF741))
        self.assertEqual(AppKit.NSExecuteFunctionKey, chr(0xF742))
        self.assertEqual(AppKit.NSUndoFunctionKey, chr(0xF743))
        self.assertEqual(AppKit.NSRedoFunctionKey, chr(0xF744))
        self.assertEqual(AppKit.NSFindFunctionKey, chr(0xF745))
        self.assertEqual(AppKit.NSHelpFunctionKey, chr(0xF746))
        self.assertEqual(AppKit.NSModeSwitchFunctionKey, chr(0xF747))

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSEventTypeGesture, 29)
        self.assertEqual(AppKit.NSEventTypeMagnify, 30)
        self.assertEqual(AppKit.NSEventTypeSwipe, 31)
        self.assertEqual(AppKit.NSEventTypeRotate, 18)
        self.assertEqual(AppKit.NSEventTypeBeginGesture, 19)
        self.assertEqual(AppKit.NSEventTypeEndGesture, 20)

        self.assertEqual(AppKit.NSEventMaskGesture, 1 << 29)
        self.assertEqual(AppKit.NSEventMaskMagnify, 1 << 30)
        self.assertEqual(AppKit.NSEventMaskSwipe, 1 << 31)
        self.assertEqual(AppKit.NSEventMaskRotate, 1 << 18)
        self.assertEqual(AppKit.NSEventMaskBeginGesture, 1 << 19)
        self.assertEqual(AppKit.NSEventMaskEndGesture, 1 << 20)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSEventPhaseNone, 0)
        self.assertEqual(AppKit.NSEventPhaseBegan, 1)
        self.assertEqual(AppKit.NSEventPhaseStationary, 2)
        self.assertEqual(AppKit.NSEventPhaseChanged, 4)
        self.assertEqual(AppKit.NSEventPhaseEnded, 8)
        self.assertEqual(AppKit.NSEventPhaseCancelled, 16)
        self.assertEqual(AppKit.NSEventPhaseMayBegin, 32)

        self.assertEqual(AppKit.NSEventGestureAxisNone, 0)
        self.assertEqual(AppKit.NSEventGestureAxisHorizontal, 1)
        self.assertEqual(AppKit.NSEventGestureAxisVertical, 2)

        self.assertEqual(AppKit.NSEventSwipeTrackingLockDirection, 1)
        self.assertEqual(AppKit.NSEventSwipeTrackingClampGestureAmount, 2)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(AppKit.NSEventTypeSmartMagnify, 32)
        self.assertEqual(AppKit.NSEventTypeQuickLook, 33)
        self.assertEqual(AppKit.NSEventMaskSmartMagnify, 1 << 32)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NSEventTypePressure, 34)
        self.assertEqual(AppKit.NSEventTypeDirectTouch, 37)
        self.assertEqual(AppKit.NSEventMaskPressure, 1 << 34)
        self.assertEqual(AppKit.NSEventMaskDirectTouch, 1 << 37)

        self.assertEqual(AppKit.NSPressureBehaviorUnknown, -1)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryDefault, 0)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryClick, 1)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryGeneric, 2)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryAccelerator, 3)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryDeepClick, 5)
        self.assertEqual(AppKit.NSPressureBehaviorPrimaryDeepDrag, 6)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertEqual(AppKit.NSEventTypeChangeMode, 38)
        self.assertEqual(
            AppKit.NSEventMaskChangeMode, 1 << AppKit.NSEventTypeChangeMode
        )

    def testFunctions(self):
        v = AppKit.NSEventMaskFromType(AppKit.NSLeftMouseDown)
        self.assertEqual(v, AppKit.NSLeftMouseDownMask)

        v = AppKit.NSEventMaskFromType(AppKit.NSOtherMouseDown)
        self.assertEqual(v, AppKit.NSOtherMouseDownMask)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(AppKit.NSEvent.isMouseCoalescingEnabled)
        self.assertArgIsBOOL(AppKit.NSEvent.setMouseCoalescingEnabled_, 0)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSEvent.isARepeat)
        self.assertResultIsBOOL(AppKit.NSEvent.isEnteringProximity)

        self.assertArgIsBOOL(
            AppKit.NSEvent.keyEventWithType_location_modifierFlags_timestamp_windowNumber_context_characters_charactersIgnoringModifiers_isARepeat_keyCode_,  # noqa: B950
            8,
        )
        self.assertArgHasType(
            AppKit.NSEvent.enterExitEventWithType_location_modifierFlags_timestamp_windowNumber_context_eventNumber_trackingNumber_userData_,  # noqa: B950
            8,
            b"^v",
        )

        self.assertResultHasType(AppKit.NSEvent.userData, b"^v")

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            AppKit.NSEvent.addGlobalMonitorForEventsMatchingMask_handler_, 1, b"v@"
        )
        self.assertArgIsBlock(
            AppKit.NSEvent.addLocalMonitorForEventsMatchingMask_handler_, 1, b"@@"
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSEvent.hasPreciseScrollingDeltas)
        self.assertResultIsBOOL(AppKit.NSEvent.isDirectionInvertedFromDevice)
        self.assertResultIsBOOL(AppKit.NSEvent.isSwipeTrackingFromScrollEventsEnabled)

        self.assertArgIsBlock(
            AppKit.NSEvent.trackSwipeEventWithOptions_dampenAmountThresholdMin_max_usingHandler_,  # noqa: B950
            3,
            b"v"
            + objc._C_CGFloat
            + objc._C_NSUInteger
            + objc._C_NSBOOL
            + b"o^"
            + objc._C_NSBOOL,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSSpeechRecognizer(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSpeechRecognizer.listensInForegroundOnly)
        self.assertResultIsBOOL(AppKit.NSSpeechRecognizer.blocksOtherRecognizers)
        self.assertArgIsBOOL(AppKit.NSSpeechRecognizer.setListensInForegroundOnly_, 0)
        self.assertArgIsBOOL(AppKit.NSSpeechRecognizer.setBlocksOtherRecognizers_, 0)

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSSpeechRecognizerDelegate")



import CoreFoundation
from PyObjCTools.TestSupport import TestCase
import objc


class TestUserNotification(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFUserNotificationRef)

    def testTypeID(self):
        value = CoreFoundation.CFUserNotificationGetTypeID()
        self.assertIsInstance(value, int)

    def testCreation(self):
        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        runloop_mode = "pyobjctest.cfusernotificaton"

        rl = CoreFoundation.CFRunLoopGetCurrent()

        infoDict = {
            CoreFoundation.kCFUserNotificationAlertHeaderKey: b"Alert Header".decode(
                "ascii"
            ),
            CoreFoundation.kCFUserNotificationProgressIndicatorValueKey: True,
            CoreFoundation.kCFUserNotificationDefaultButtonTitleKey: "Cancel",
        }

        self.assertArgIsOut(CoreFoundation.CFUserNotificationCreate, 3)
        ref, error = CoreFoundation.CFUserNotificationCreate(
            None, 3.0, 0, None, infoDict
        )
        self.assertEqual(error, 0)
        self.assertIsInstance(ref, CoreFoundation.CFUserNotificationRef)
        values = []

        @objc.callbackFor(CoreFoundation.CFUserNotificationCreateRunLoopSource)
        def callout(notification, flags):
            values.append((notification, flags))
            CoreFoundation.CFRunLoopStop(rl)

        self.assertArgIsFunction(
            CoreFoundation.CFUserNotificationCreateRunLoopSource,
            2,
            b"v^{__CFUserNotification=}" + objc._C_NSInteger,
            True,
        )
        rls = CoreFoundation.CFUserNotificationCreateRunLoopSource(
            None, ref, callout, 1
        )
        self.assertIsInstance(rls, CoreFoundation.CFRunLoopSourceRef)
        CoreFoundation.CFRunLoopAddSource(rl, rls, runloop_mode)
        CoreFoundation.CFRunLoopRunInMode(runloop_mode, 5.0, True)

        CoreFoundation.CFUserNotificationCancel(ref)
        CoreFoundation.CFRunLoopRunInMode(runloop_mode, 5.0, True)
        CoreFoundation.CFRunLoopRemoveSource(rl, rls, runloop_mode)

        self.assertEqual(len(values), 1)
        self.assertIs(values[0][0], ref)
        self.assertIsInstance(values[0][1], int)

        self.assertArgIsOut(CoreFoundation.CFUserNotificationReceiveResponse, 2)
        error, flags = CoreFoundation.CFUserNotificationReceiveResponse(ref, 1.0, None)
        self.assertIsInstance(error, int)
        self.assertIsInstance(flags, int)
        v = CoreFoundation.CFUserNotificationGetResponseDictionary(ref)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        error = CoreFoundation.CFUserNotificationUpdate(ref, 2.0, 0, infoDict)
        self.assertEqual(error, 0)
        error = CoreFoundation.CFUserNotificationCancel(ref)
        self.assertEqual(error, 0)

        v = CoreFoundation.CFUserNotificationGetResponseValue(
            ref, CoreFoundation.kCFUserNotificationTextFieldValuesKey, 0
        )
        if v is not None:
            self.assertIsInstance(v, str)

    def testAlert(self):
        err, flags = CoreFoundation.CFUserNotificationDisplayAlert(
            0.1, 0, None, None, None, "Header", "Message", "Cancel", None, None, None
        )
        self.assertEqual(err, 0)
        self.assertIsInstance(flags, int)
        err, flags = CoreFoundation.CFUserNotificationDisplayAlert(
            0.1, 0, None, None, None, "Header", "Message", "Cancel", "OK", "Rest", None
        )
        self.assertEqual(err, 0)
        self.assertIsInstance(flags, int)

    def testNotice(self):
        err = CoreFoundation.CFUserNotificationDisplayNotice(
            0.1, 0, None, None, None, "Header", "Message", "Cancel"
        )
        self.assertEqual(err, 0)

    def testInlines(self):
        flag = CoreFoundation.CFUserNotificationCheckBoxChecked(2)
        self.assertEqual(flag, 1 << 10)
        flag = CoreFoundation.CFUserNotificationSecureTextField(2)
        self.assertEqual(flag, 1 << 18)
        flag = CoreFoundation.CFUserNotificationPopUpSelection(2)
        self.assertEqual(flag, 2 << 24)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFUserNotificationStopAlertLevel, 0)
        self.assertEqual(CoreFoundation.kCFUserNotificationNoteAlertLevel, 1)
        self.assertEqual(CoreFoundation.kCFUserNotificationCautionAlertLevel, 2)
        self.assertEqual(CoreFoundation.kCFUserNotificationPlainAlertLevel, 3)
        self.assertEqual(CoreFoundation.kCFUserNotificationDefaultResponse, 0)
        self.assertEqual(CoreFoundation.kCFUserNotificationAlternateResponse, 1)
        self.assertEqual(CoreFoundation.kCFUserNotificationOtherResponse, 2)
        self.assertEqual(CoreFoundation.kCFUserNotificationCancelResponse, 3)
        self.assertEqual(
            CoreFoundation.kCFUserNotificationNoDefaultButtonFlag, (1 << 5)
        )
        self.assertEqual(
            CoreFoundation.kCFUserNotificationUseRadioButtonsFlag, (1 << 6)
        )
        self.assertIsInstance(CoreFoundation.kCFUserNotificationIconURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationSoundURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationLocalizationURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationAlertHeaderKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationAlertMessageKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFUserNotificationDefaultButtonTitleKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFUserNotificationAlternateButtonTitleKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFUserNotificationOtherButtonTitleKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFUserNotificationProgressIndicatorValueKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFUserNotificationPopUpTitlesKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationTextFieldTitlesKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationCheckBoxTitlesKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationTextFieldValuesKey, str)
        self.assertIsInstance(CoreFoundation.kCFUserNotificationPopUpSelectionKey, str)



import objc
from Foundation import NSObject
from objc import super


class Converter(NSObject):
    exchangeRate = objc.ivar.double()
    dollarsToConvert = objc.ivar.double()

    def init(self):
        self = super().init()
        self.exchangeRate = 3
        self.dollarsToConvert = 4
        return self

    def amountInOtherCurrency(self):
        return self.dollarsToConvert * self.exchangeRate


Converter.setKeys_triggerChangeNotificationsForDependentKey_(
    ["dollarsToConvert", "exchangeRate"], "amountInOtherCurrency"
)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSScriptClassDescription(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSScriptClassDescription.matchesAppleEventCode_
        )
        self.assertResultIsBOOL(Foundation.NSScriptClassDescription.supportsCommand_)
        self.assertResultIsBOOL(
            Foundation.NSScriptClassDescription.isLocationRequiredToCreateForKey_
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(Foundation.NSScriptClassDescription.hasPropertyForKey_)
        self.assertResultIsBOOL(
            Foundation.NSScriptClassDescription.hasOrderedToManyRelationshipForKey_
        )
        self.assertResultIsBOOL(
            Foundation.NSScriptClassDescription.hasReadablePropertyForKey_
        )
        self.assertResultIsBOOL(
            Foundation.NSScriptClassDescription.hasWritablePropertyForKey_
        )
        self.assertResultIsBOOL(Foundation.NSScriptClassDescription.isReadOnlyKey_)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSoundHelper(AppKit.NSObject):
    def sound_didFinishPlaying_(self, s, p):
        pass


class TestNSSound(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSSoundPboardType, str)

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSSound.initWithContentsOfURL_byReference_, 1)
        self.assertArgIsBOOL(AppKit.NSSound.initWithContentsOfFile_byReference_, 1)
        self.assertResultIsBOOL(AppKit.NSSound.setName_)
        self.assertResultIsBOOL(AppKit.NSSound.canInitWithPasteboard_)
        self.assertResultIsBOOL(AppKit.NSSound.play)
        self.assertResultIsBOOL(AppKit.NSSound.pause)
        self.assertResultIsBOOL(AppKit.NSSound.resume)
        self.assertResultIsBOOL(AppKit.NSSound.stop)
        self.assertResultIsBOOL(AppKit.NSSound.isPlaying)
        self.assertResultIsBOOL(AppKit.NSSound.loops)
        self.assertArgIsBOOL(AppKit.NSSound.setLoops_, 0)

    @min_os_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSSoundDelegate")

    def testProtocols(self):
        self.assertArgIsBOOL(TestNSSoundHelper.sound_didFinishPlaying_, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSScriptCoercionHandler(TestCase):
    def testMethods(self):
        self.assertArgIsSEL(
            Foundation.NSScriptCoercionHandler.registerCoercer_selector_toConvertFromClass_toClass_,  # noqa: B950
            1,
            b"@@:@#",
        )



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSStoryboardSegueHelper(AppKit.NSObject):
    def shouldPerformSegueWithIdentifier_sender_(self, i, s):
        return 1


class TestNSStoryboardSegue(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            AppKit.NSStoryboardSegue.segueWithIdentifier_source_destination_performHandler_,
            3,
            b"v",
        )

        self.assertArgIsBlock(AppKit.NSStoryboardSegue.setPerformHandler_, 0, b"v")
        self.assertResultIsBlock(AppKit.NSStoryboardSegue.performHandler, b"v")

    @min_os_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSSeguePerforming")

        self.assertResultIsBOOL(
            TestNSStoryboardSegueHelper.shouldPerformSegueWithIdentifier_sender_
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSDatePickerHelper(AppKit.NSObject):
    def datePickerCell_validateProposedDateValue_timeInterval_(self, v1, v2, v3):
        pass


class TestNSDatePickerCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTextFieldAndStepperDatePickerStyle, 0)
        self.assertEqual(AppKit.NSClockAndCalendarDatePickerStyle, 1)
        self.assertEqual(AppKit.NSTextFieldDatePickerStyle, 2)

        self.assertEqual(AppKit.NSSingleDateMode, 0)
        self.assertEqual(AppKit.NSRangeDateMode, 1)

        self.assertEqual(AppKit.NSHourMinuteDatePickerElementFlag, 0x000C)
        self.assertEqual(AppKit.NSHourMinuteSecondDatePickerElementFlag, 0x000E)
        self.assertEqual(AppKit.NSTimeZoneDatePickerElementFlag, 0x0010)
        self.assertEqual(AppKit.NSYearMonthDatePickerElementFlag, 0x00C0)
        self.assertEqual(AppKit.NSYearMonthDayDatePickerElementFlag, 0x00E0)
        self.assertEqual(AppKit.NSEraDatePickerElementFlag, 0x0100)

        self.assertEqual(AppKit.NSDatePickerStyleTextFieldAndStepper, 0)
        self.assertEqual(AppKit.NSDatePickerStyleClockAndCalendar, 1)
        self.assertEqual(AppKit.NSDatePickerStyleTextField, 2)

        self.assertEqual(AppKit.NSDatePickerModeSingle, 0)
        self.assertEqual(AppKit.NSDatePickerModeRange, 1)

        self.assertEqual(AppKit.NSDatePickerElementFlagHourMinute, 0x000C)
        self.assertEqual(AppKit.NSDatePickerElementFlagHourMinuteSecond, 0x000E)
        self.assertEqual(AppKit.NSDatePickerElementFlagTimeZone, 0x0010)

        self.assertEqual(AppKit.NSDatePickerElementFlagYearMonth, 0x00C0)
        self.assertEqual(AppKit.NSDatePickerElementFlagYearMonthDay, 0x00E0)
        self.assertEqual(AppKit.NSDatePickerElementFlagEra, 0x0100)

    def testMethods(self):
        o = TestNSDatePickerHelper.alloc().init()
        m = o.datePickerCell_validateProposedDateValue_timeInterval_.__metadata__()
        self.assertEqual(m["arguments"][3]["type"], b"N^@")
        self.assertStartswith(m["arguments"][4]["type"], b"N^")

        self.assertResultIsBOOL(AppKit.NSDatePickerCell.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSDatePickerCell.setDrawsBackground_, 0)

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSDatePickerCellDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTintConfiguration(TestCase):
    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertResultIsBOOL(AppKit.NSTintConfiguration.adaptsToUserAccentColor)



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNumberFormatter(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFNumberFormatterRef)

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFNumberFormatterGetTypeID(), int)

    def testFuncs(self):
        locale = CoreFoundation.CFLocaleCreate(None, "en_US")
        fmt = CoreFoundation.CFNumberFormatterCreate(
            None, locale, CoreFoundation.kCFNumberFormatterDecimalStyle
        )
        self.assertIsInstance(fmt, CoreFoundation.CFNumberFormatterRef)
        v = CoreFoundation.CFNumberFormatterGetLocale(fmt)
        self.assertIs(v, locale)
        v = CoreFoundation.CFNumberFormatterGetStyle(fmt)
        self.assertEqual(v, CoreFoundation.kCFNumberFormatterDecimalStyle)
        v = CoreFoundation.CFNumberFormatterGetFormat(fmt)
        self.assertIsInstance(v, str)
        CoreFoundation.CFNumberFormatterSetFormat(fmt, v[:-2])
        v2 = CoreFoundation.CFNumberFormatterGetFormat(fmt)
        self.assertEqual(v2, v[:-2])
        v = CoreFoundation.CFNumberFormatterCreateStringWithNumber(None, fmt, 42.5)
        self.assertIsInstance(v, str)
        self.assertEqual(v, "42.5")

        v = CoreFoundation.CFNumberFormatterCreateStringWithValue(
            None, fmt, CoreFoundation.kCFNumberDoubleType, 42.5
        )
        self.assertIsInstance(v, str)
        self.assertEqual(v, "42.5")
        num, rng = CoreFoundation.CFNumberFormatterCreateNumberFromString(
            None, fmt, "42.0a", (0, 5), 0
        )
        self.assertEqual(num, 42.0)
        self.assertEqual(rng, (0, 4))
        ok, rng, num = CoreFoundation.CFNumberFormatterGetValueFromString(
            fmt, "42.0a", (0, 5), CoreFoundation.kCFNumberDoubleType, None
        )
        self.assertEqual(ok, True)
        self.assertEqual(num, 42.0)
        self.assertEqual(rng, (0, 4))
        num, rng = CoreFoundation.CFNumberFormatterCreateNumberFromString(
            None,
            fmt,
            "42.0a",
            (0, 5),
            CoreFoundation.kCFNumberFormatterParseIntegersOnly,
        )
        self.assertEqual(num, 42)
        self.assertEqual(rng, (0, 2))
        v = CoreFoundation.CFNumberFormatterCopyProperty(
            fmt, CoreFoundation.kCFNumberFormatterCurrencyCode
        )
        self.assertIsInstance(v, str)
        CoreFoundation.CFNumberFormatterSetProperty(
            fmt, CoreFoundation.kCFNumberFormatterCurrencyCode, "HFL"
        )

        self.assertResultIsCFRetained(CoreFoundation.CFNumberFormatterCopyProperty)
        v = CoreFoundation.CFNumberFormatterCopyProperty(
            fmt, CoreFoundation.kCFNumberFormatterCurrencyCode
        )
        self.assertEqual(v, "HFL")
        self.assertArgIsOut(
            CoreFoundation.CFNumberFormatterGetDecimalInfoForCurrencyCode, 1
        )
        self.assertArgIsOut(
            CoreFoundation.CFNumberFormatterGetDecimalInfoForCurrencyCode, 2
        )
        ok, frac, rnd = CoreFoundation.CFNumberFormatterGetDecimalInfoForCurrencyCode(
            "EUR", None, None
        )
        self.assertEqual(ok, True)
        self.assertEqual(frac, 2)
        self.assertEqual(rnd, 0.0)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFNumberFormatterNoStyle, 0)
        self.assertEqual(CoreFoundation.kCFNumberFormatterDecimalStyle, 1)
        self.assertEqual(CoreFoundation.kCFNumberFormatterCurrencyStyle, 2)
        self.assertEqual(CoreFoundation.kCFNumberFormatterPercentStyle, 3)
        self.assertEqual(CoreFoundation.kCFNumberFormatterScientificStyle, 4)
        self.assertEqual(CoreFoundation.kCFNumberFormatterSpellOutStyle, 5)

        self.assertEqual(CoreFoundation.kCFNumberFormatterParseIntegersOnly, 1)

        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundCeiling, 0)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundFloor, 1)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundDown, 2)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundUp, 3)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundHalfEven, 4)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundHalfDown, 5)
        self.assertEqual(CoreFoundation.kCFNumberFormatterRoundHalfUp, 6)

        self.assertEqual(CoreFoundation.kCFNumberFormatterPadBeforePrefix, 0)
        self.assertEqual(CoreFoundation.kCFNumberFormatterPadAfterPrefix, 1)
        self.assertEqual(CoreFoundation.kCFNumberFormatterPadBeforeSuffix, 2)
        self.assertEqual(CoreFoundation.kCFNumberFormatterPadAfterSuffix, 3)

        self.assertIsInstance(CoreFoundation.kCFNumberFormatterCurrencyCode, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterDecimalSeparator, str)
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterCurrencyDecimalSeparator, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterAlwaysShowDecimalSeparator, str
        )
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterGroupingSeparator, str)
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterUseGroupingSeparator, str
        )
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPercentSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterZeroSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterNaNSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterInfinitySymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMinusSign, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPlusSign, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterCurrencySymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterExponentSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMinIntegerDigits, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMaxIntegerDigits, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMinFractionDigits, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMaxFractionDigits, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterGroupingSize, str)
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterSecondaryGroupingSize, str
        )
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterRoundingMode, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterRoundingIncrement, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterFormatWidth, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPaddingPosition, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPaddingCharacter, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterDefaultFormat, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterMultiplier, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPositivePrefix, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPositiveSuffix, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterNegativePrefix, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterNegativeSuffix, str)
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterPerMillSymbol, str)
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterInternationalCurrencySymbol, str
        )

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterCurrencyGroupingSeparator, str
        )
        self.assertIsInstance(CoreFoundation.kCFNumberFormatterIsLenient, str)
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterUseSignificantDigits, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterMinSignificantDigits, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFNumberFormatterMaxSignificantDigits, str
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(CoreFoundation.kCFNumberFormatterOrdinalStyle, 6)
        self.assertEqual(CoreFoundation.kCFNumberFormatterCurrencyISOCodeStyle, 8)
        self.assertEqual(CoreFoundation.kCFNumberFormatterCurrencyPluralStyle, 9)
        self.assertEqual(CoreFoundation.kCFNumberFormatterCurrencyAccountingStyle, 10)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSAnimationHelper(AppKit.NSObject):
    def animationShouldStart_(self, animation):
        return 1

    def animation_valueForProgress_(self, a, b):
        return 1

    def animation_didReachProgressMark_(self, a, b):
        return 1


class TestNSAnimation(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSAnimationEaseInOut, 0)
        self.assertEqual(AppKit.NSAnimationEaseIn, 1)
        self.assertEqual(AppKit.NSAnimationEaseOut, 2)
        self.assertEqual(AppKit.NSAnimationLinear, 3)

        self.assertEqual(AppKit.NSAnimationBlocking, 0)
        self.assertEqual(AppKit.NSAnimationNonblocking, 1)
        self.assertEqual(AppKit.NSAnimationNonblockingThreaded, 2)

        self.assertIsInstance(AppKit.NSAnimationProgressMarkNotification, str)
        self.assertIsInstance(AppKit.NSAnimationProgressMark, str)

        self.assertIsInstance(AppKit.NSViewAnimationTargetKey, str)
        self.assertIsInstance(AppKit.NSViewAnimationStartFrameKey, str)
        self.assertIsInstance(AppKit.NSViewAnimationEndFrameKey, str)
        self.assertIsInstance(AppKit.NSViewAnimationEffectKey, str)
        self.assertIsInstance(AppKit.NSViewAnimationFadeInEffect, str)
        self.assertIsInstance(AppKit.NSViewAnimationFadeOutEffect, str)

        self.assertIsInstance(AppKit.NSAnimationTriggerOrderIn, str)
        self.assertIsInstance(AppKit.NSAnimationTriggerOrderOut, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSAnimation.isAnimating)

    @min_sdk_level("10.6")
    def testProtocol(self):
        objc.protocolNamed("NSAnimationDelegate")
        objc.protocolNamed("NSAnimatablePropertyContainer")

    def testProtocolMethods(self):
        self.assertResultIsBOOL(TestNSAnimationHelper.animationShouldStart_)

        self.assertResultHasType(
            TestNSAnimationHelper.animation_valueForProgress_, objc._C_FLT
        )
        self.assertArgHasType(
            TestNSAnimationHelper.animation_valueForProgress_, 1, objc._C_FLT
        )
        self.assertArgHasType(
            TestNSAnimationHelper.animation_didReachProgressMark_, 1, objc._C_FLT
        )



#
#  ToDosDocument.py
#  ToDos
#
#  Converted by u.fiedler on 09.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Category import Category
from Cocoa import (
    NSDocument,
    NSKeyedArchiver,
    NSKeyedUnarchiver,
    NSMutableArray,
    NSValueTransformer,
)
from objc import super
from OverdueTransformer import OverdueTransformer
from PriorityToColourTransformer import PriorityToColourTransformer


class ToDosDocument(NSDocument):
    nix = objc.IBOutlet()
    toDos = objc.ivar()

    def init(self):
        self = super().init()
        if self is None:
            return None
        self.toDos = NSMutableArray()
        return self  # if this line is missing you will get the
        # simple message: "Can't create new document"

    def categories(self):
        return Category.allCategories()

    def windowNibName(self):
        return "ToDosDocument"

    def dataRepresentationOfType_(self, aType):
        return NSKeyedArchiver.archivedDataWithRootObject_(self.toDos)

    def loadDataRepresentation_ofType_(self, data, aType):
        self.toDos = NSKeyedUnarchiver.unarchiveObjectWithData_(data)
        return True


priorityTransformer = PriorityToColourTransformer.alloc().init()
NSValueTransformer.setValueTransformer_forName_(
    priorityTransformer, "PriorityToColourTransformer"
)

overdueTransformer = OverdueTransformer.alloc().init()
NSValueTransformer.setValueTransformer_forName_(
    overdueTransformer, "OverdueTransformer"
)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSActionCell(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSActionCell.setBordered_, 0)
        self.assertArgIsBOOL(AppKit.NSActionCell.setBezeled_, 0)
        self.assertArgIsBOOL(AppKit.NSActionCell.setEnabled_, 0)
        self.assertArgIsBOOL(AppKit.NSActionCell.setFloatingPointFormat_left_right_, 0)
        self.assertArgIsSEL(AppKit.NSActionCell.setAction_, 0, b"v@:@")



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestAttributedString(TestCase):
    def testTypes(self):
        try:
            NSCFAttributedString = objc.lookUpClass("__NSCFAttributedString")
        except objc.error:
            NSCFAttributedString = objc.lookUpClass("NSCFAttributedString")

        self.assertIs(CoreFoundation.CFAttributedStringRef, NSCFAttributedString)
        self.assertIs(CoreFoundation.CFMutableAttributedStringRef, NSCFAttributedString)

    def testTypeID(self):
        v = CoreFoundation.CFAttributedStringGetTypeID()
        self.assertIsInstance(v, int)

    def testCreate(self):
        val = CoreFoundation.CFAttributedStringCreate(None, "hello", {"foo": 42})
        self.assertIsInstance(val, CoreFoundation.CFAttributedStringRef)
        val = CoreFoundation.CFAttributedStringCreateWithSubstring(None, val, (1, 2))
        self.assertIsInstance(val, CoreFoundation.CFAttributedStringRef)
        val2 = CoreFoundation.CFAttributedStringCreateCopy(None, val)
        self.assertIs(val2, val)

    def testGetting(self):
        val = CoreFoundation.CFAttributedStringCreate(
            None, "hello", {"foo": 42, "bar": b"baz"}
        )
        self.assertIsInstance(val, CoreFoundation.CFAttributedStringRef)
        dta = CoreFoundation.CFAttributedStringGetString(val)
        self.assertEqual(dta, "hello")
        length = CoreFoundation.CFAttributedStringGetLength(val)
        self.assertEqual(length, 5)
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 1, None)
        self.assertEqual(v, {"foo": 42, "bar": b"baz"})
        self.assertEqual(rng, (0, 5))
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 1, objc.NULL)
        self.assertEqual(v, {"foo": 42, "bar": b"baz"})
        self.assertEqual(rng, objc.NULL)
        v, rng = CoreFoundation.CFAttributedStringGetAttribute(val, 1, "foo", None)
        self.assertEqual(v, 42)
        self.assertEqual(rng, (0, 5))
        v, rng = CoreFoundation.CFAttributedStringGetAttribute(val, 1, "foo", objc.NULL)
        self.assertEqual(v, 42)
        self.assertEqual(rng, objc.NULL)
        v, rng = CoreFoundation.CFAttributedStringGetAttributesAndLongestEffectiveRange(
            val, 1, (0, 5), None
        )
        self.assertEqual(v, {"foo": 42, "bar": b"baz"})
        self.assertEqual(rng, (0, 5))
        v, rng = CoreFoundation.CFAttributedStringGetAttributesAndLongestEffectiveRange(
            val, 1, (0, 5), objc.NULL
        )
        self.assertEqual(v, {"foo": 42, "bar": b"baz"})
        self.assertEqual(rng, objc.NULL)
        v, rng = CoreFoundation.CFAttributedStringGetAttributeAndLongestEffectiveRange(
            val, 1, "bar", (0, 5), None
        )
        self.assertEqual(v, b"baz")
        self.assertEqual(rng, (0, 5))
        v, rng = CoreFoundation.CFAttributedStringGetAttributeAndLongestEffectiveRange(
            val, 1, "bar", (0, 5), objc.NULL
        )
        self.assertEqual(v, b"baz")
        self.assertEqual(rng, objc.NULL)

    def testMutableCopy(self):
        val = CoreFoundation.CFAttributedStringCreateMutable(None, 0)
        self.assertIsInstance(val, CoreFoundation.CFAttributedStringRef)
        orig = CoreFoundation.CFAttributedStringCreate(
            None, "hello", {"foo": 42, "bar": "baz"}
        )
        self.assertIsInstance(orig, CoreFoundation.CFAttributedStringRef)
        val = CoreFoundation.CFAttributedStringCreateMutableCopy(None, 0, orig)
        self.assertIsInstance(orig, CoreFoundation.CFAttributedStringRef)
        self.assertIsNot(val, orig)
        CoreFoundation.CFAttributedStringReplaceString(val, (0, 3), "Hal")
        dta = CoreFoundation.CFAttributedStringGetString(val)
        self.assertEqual(dta, "Hallo")
        v = CoreFoundation.CFAttributedStringGetMutableString(val)
        self.assertIs(v, None)
        CoreFoundation.CFAttributedStringSetAttributes(
            val, (0, 2), {"ronald": 99}, False
        )
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 1, None)
        self.assertEqual(v, {"ronald": 99, "foo": 42, "bar": "baz"})
        self.assertEqual(rng, (0, 2))
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 3, None)
        self.assertEqual(v, {"foo": 42, "bar": "baz"})
        self.assertEqual(rng, (2, 3))
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        CoreFoundation.CFAttributedStringSetAttributes(
            val, (0, 2), {"ronald": 99}, True
        )
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 1, None)
        self.assertEqual(v, {"ronald": 99})
        self.assertEqual(rng, (0, 2))
        CoreFoundation.CFAttributedStringSetAttribute(val, (1, 3), "color", "blue")
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 1, None)
        self.assertEqual(v, {"ronald": 99, "color": "blue"})
        self.assertEqual(rng, (1, 1))
        CoreFoundation.CFAttributedStringRemoveAttribute(val, (1, 3), "color")
        v, rng = CoreFoundation.CFAttributedStringGetAttributes(val, 3, None)
        self.assertEqual(v, {"foo": 42, "bar": "baz"})
        self.assertEqual(rng, (2, 2))
        rep = CoreFoundation.CFAttributedStringCreate(None, "dummy", {"attrib": 99})
        CoreFoundation.CFAttributedStringReplaceAttributedString(val, (1, 3), rep)
        self.assertEqual(CoreFoundation.CFAttributedStringGetString(val), "Hdummyo")

    def testEditing(self):
        val = CoreFoundation.CFAttributedStringCreateMutable(None, 0)
        self.assertIsInstance(val, CoreFoundation.CFAttributedStringRef)
        CoreFoundation.CFAttributedStringBeginEditing(val)
        CoreFoundation.CFAttributedStringEndEditing(val)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSplitViewController(TestCase):
    @min_os_level("10.10")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSplitViewItem.isCollapsed)
        self.assertArgIsBOOL(AppKit.NSSplitViewItem.setCollapsed_, 0)
        self.assertResultIsBOOL(AppKit.NSSplitViewItem.canCollapse)
        self.assertArgIsBOOL(AppKit.NSSplitViewItem.setCanCollapse_, 0)

        self.assertResultIsBOOL(
            AppKit.NSSplitViewController.splitView_canCollapseSubview_
        )
        self.assertResultIsBOOL(
            AppKit.NSSplitViewController.splitView_shouldCollapseSubview_forDoubleClickOnDividerAtIndex_  # noqa: B950
        )
        self.assertResultIsBOOL(
            AppKit.NSSplitViewController.splitView_shouldHideDividerAtIndex_
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSSplitViewItem.isSpringLoaded)
        self.assertArgIsBOOL(AppKit.NSSplitViewItem.setSpringLoaded_, 0)
        self.assertResultIsBOOL(AppKit.NSSplitViewController.validateUserInterfaceItem_)

    def testConstants(self):
        self.assertEqual(AppKit.NSSplitViewItemBehaviorDefault, 0)
        self.assertEqual(AppKit.NSSplitViewItemBehaviorSidebar, 1)
        self.assertEqual(AppKit.NSSplitViewItemBehaviorContentList, 2)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSSplitViewControllerAutomaticDimension, float)
        self.assertIsInstance(AppKit.NSSplitViewItemUnspecifiedDimension, float)

        self.assertEqual(AppKit.NSSplitViewItemCollapseBehaviorDefault, 0)
        self.assertEqual(
            AppKit.NSSplitViewItemCollapseBehaviorPreferResizingSplitViewWithFixedSiblings,
            1,
        )
        self.assertEqual(
            AppKit.NSSplitViewItemCollapseBehaviorPreferResizingSiblingsWithFixedSplitView,
            2,
        )
        self.assertEqual(
            AppKit.NSSplitViewItemCollapseBehaviorUseConstraints, 3
        )  # noqa: B950



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSTableViewDiffableDataSource(TestCase):
    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertArgIsBlock(
            AppKit.NSTableViewDiffableDataSource.initWithTableView_cellProvider_,
            1,
            b"@@@" + objc._C_NSInteger + b"@",
        )

        self.assertArgIsBOOL(
            AppKit.NSTableViewDiffableDataSource.applySnapshot_animatingDifferences_, 1
        )

        self.assertArgIsBOOL(
            AppKit.NSTableViewDiffableDataSource.applySnapshot_animatingDifferences_completion_,
            1,
        )
        self.assertArgIsBlock(
            AppKit.NSTableViewDiffableDataSource.applySnapshot_animatingDifferences_completion_,
            2,
            b"v",
        )

        self.assertResultIsBlock(
            AppKit.NSTableViewDiffableDataSource.rowViewProvider,
            b"@@" + objc._C_NSInteger + b"@",
        )
        self.assertArgIsBlock(
            AppKit.NSTableViewDiffableDataSource.setRowViewProvider_,
            0,
            b"@@" + objc._C_NSInteger + b"@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextAlternatives(TestCase):
    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(
            AppKit.NSTextAlternativesSelectedAlternativeStringNotification, str
        )



import objc
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSetInteraction(TestCase):
    def testRepeatedAllocInit(self):
        for _ in range(1, 1000):
            _ = Foundation.NSSet.alloc().init()

    def testContains(self):
        x = Foundation.NSSet.setWithArray_(["foo", "bar", "baz"])

        self.assertIn("foo", x)
        self.assertNotIn("notfoo", x)

    def testIteration(self):
        x = Foundation.NSSet.setWithArray_(["foo", "bar", "baz"])

        for i in x:
            self.assertIn(i, x)
            self.assertTrue(x.containsObject_(i))

    def test_varargsConstruction(self):
        w = Foundation.NSSet.setWithObjects_(0, 1, 2, 3, None)
        x = Foundation.NSSet.alloc().initWithObjects_(0, 1, 2, 3, None)
        y = Foundation.NSSet.setWithObjects_count_(range(10), 4)
        z = Foundation.NSSet.alloc().initWithObjects_count_(range(10), 4)
        # a = Foundation.NSSet.alloc().initWithObjects_count_(range(4), None)

        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)
        # self.assertEqual(len(a), 4)

        self.assertIn(0, w)
        self.assertIn(1, x)
        self.assertIn(2, y)
        self.assertIn(3, z)
        # self.assertIn(3, a)

    def test_varargsConstruction2(self):
        w = Foundation.NSMutableSet.setWithObjects_(0, 1, 2, 3, None)
        x = Foundation.NSMutableSet.alloc().initWithObjects_(0, 1, 2, 3, None)
        y = Foundation.NSMutableSet.setWithObjects_count_(range(10), 4)
        z = Foundation.NSMutableSet.alloc().initWithObjects_count_(range(10), 4)

        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)

        self.assertIn(0, w)
        self.assertIn(1, x)
        self.assertIn(2, y)
        self.assertIn(3, z)


class TestVariadic(TestCase):
    def testSetWithObjects(self):
        o = Foundation.NSSet.setWithObjects_()
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSSet)

        o = Foundation.NSSet.setWithObjects_(1, 2, 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

        o = Foundation.NSMutableSet.setWithObjects_()
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSMutableSet)

        o = Foundation.NSMutableSet.setWithObjects_(1, 2, 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSMutableSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

    def testInitWithObjects(self):
        o = Foundation.NSSet.alloc().initWithObjects_()
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSSet)

        o = Foundation.NSSet.alloc().initWithObjects_(1, 2, 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

        o = Foundation.NSMutableSet.alloc().initWithObjects_()
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSMutableSet)

        o = Foundation.NSMutableSet.alloc().initWithObjects_(1, 2, 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSMutableSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

    def testSetWithObjectsCount(self):
        o = Foundation.NSSet.setWithObjects_count_([1, 2, 3], 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)
        self.assertNotIn(4, o)

        o = Foundation.NSSet.setWithObjects_count_([1, 2, 3], 0)
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSSet)

        o = Foundation.NSMutableSet.setWithObjects_count_([1, 2, 3], 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSMutableSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

        o = Foundation.NSMutableSet.setWithObjects_count_([1, 2, 3], 0)
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSMutableSet)

    def testInitWithObjectsCount(self):
        o = Foundation.NSSet.alloc().initWithObjects_count_([1, 2, 3], 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)
        self.assertNotIn(4, o)

        o = Foundation.NSSet.alloc().initWithObjects_count_([1, 2, 3], 0)
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSSet)

        o = Foundation.NSMutableSet.alloc().initWithObjects_count_([1, 2, 3], 3)
        self.assertEqual(len(o), 3)
        self.assertIsInstance(o, Foundation.NSMutableSet)
        self.assertIn(1, o)
        self.assertIn(2, o)
        self.assertIn(3, o)

        o = Foundation.NSMutableSet.alloc().initWithObjects_count_([1, 2, 3], 0)
        self.assertEqual(len(o), 0)
        self.assertIsInstance(o, Foundation.NSMutableSet)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSSet.containsObject_)
        self.assertResultIsBOOL(Foundation.NSSet.intersectsSet_)
        self.assertResultIsBOOL(Foundation.NSSet.isEqualToSet_)
        self.assertResultIsBOOL(Foundation.NSSet.isSubsetOfSet_)

        self.assertArgIsIn(Foundation.NSSet.setWithObjects_count_, 0)
        self.assertArgSizeInArg(Foundation.NSSet.setWithObjects_count_, 0, 1)
        self.assertArgIsIn(Foundation.NSSet.initWithObjects_count_, 0)
        self.assertArgSizeInArg(Foundation.NSSet.initWithObjects_count_, 0, 1)

        self.assertArgIsBOOL(Foundation.NSSet.initWithSet_copyItems_, 1)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSSet.enumerateObjectsUsingBlock_, 0, b"v@o^" + objc._C_NSBOOL
        )
        self.assertArgIsBlock(
            Foundation.NSSet.enumerateObjectsWithOptions_usingBlock_,
            1,
            b"v@o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSSet.objectsPassingTest_,
            0,
            objc._C_NSBOOL + b"@o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSSet.objectsWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@o^" + objc._C_NSBOOL,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSLevelIndicatorCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSRelevancyLevelIndicatorStyle, 0)
        self.assertEqual(AppKit.NSContinuousCapacityLevelIndicatorStyle, 1)
        self.assertEqual(AppKit.NSDiscreteCapacityLevelIndicatorStyle, 2)
        self.assertEqual(AppKit.NSRatingLevelIndicatorStyle, 3)

        self.assertEqual(AppKit.NSLevelIndicatorStyleRelevancy, 0)
        self.assertEqual(AppKit.NSLevelIndicatorStyleContinuousCapacity, 1)
        self.assertEqual(AppKit.NSLevelIndicatorStyleDiscreteCapacity, 2)
        self.assertEqual(AppKit.NSLevelIndicatorStyleRating, 3)



import Cocoa
import objc
from MyWindowController import MyWindowController


class AppDelegate(Cocoa.NSObject):
    myWindowController = objc.ivar()

    @objc.IBAction
    def newDocument_(self, sender):
        if self.myWindowController is None:
            self.myWindowController = MyWindowController.alloc().initWithWindowNibName_(
                "TestWindow"
            )

        self.myWindowController.showWindow_(self)

    def applicationDidFinishLaunching_(self, notification):
        self.newDocument_(self)

    def validateMenuItem_(self, theMenuItem):
        enable = self.respondsToSelector_(theMenuItem.action())

        # disable "New" if the window is already up
        if theMenuItem.action() == b"newDocument:":
            if self.myWindowController.window().isKeyWindow():
                enable = False

        return enable



#
#  TransformerAppDelegate.py
#  Transformer
#
#  Converted by ufiedler@web.de on 03.02.05.
#
#  Value Transformer Example Application
#  Based on Apples FahrenheitToCelsiusTransformer Example

import objc
from Cocoa import NSNumber, NSObject, NSValueTransformer


class FahrenheitToCelsiusTransformer(NSValueTransformer):
    # While not strictly necessary, because PyObjC can deduce that
    # these selectors should be implemented for the class,
    # declaring them as classmethod helps make this more clear.

    @classmethod
    def transformedValueClass(cls):
        return NSNumber

    @classmethod
    def allowsReverseTransformation(cls):
        return True

    def transformedValue_(self, value):
        if value is None:
            return None
        # the forward value is coming from an ivar that's
        # already a number, so we don't need to coerce this...
        # but we do it anyway for
        fahrenheitInputValue = float(value)
        # calculate Celsius value
        celsiusOutputValue = (5.0 / 9.0) * (fahrenheitInputValue - 32.0)
        return celsiusOutputValue

    def reverseTransformedValue_(self, value):
        if value is None:
            return None
        # the reverse value comes from the text field, so it's
        # going to be a string that we need to coerce to float.
        celsiusInputValue = float(value)
        # calculate Fahrenheit value
        fahrenheitOutputValue = ((9.0 / 5.0) * celsiusInputValue) + 32.0
        return fahrenheitOutputValue


class TransformerAppDelegate(NSObject):
    fahrenheit = objc.ivar("fahrenheit", objc._C_DBL)


trans = FahrenheitToCelsiusTransformer.alloc().init()
NSValueTransformer.setValueTransformer_forName_(trans, "FahrenheitToCelsiusTransformer")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSToolbarItemHelper(AppKit.NSObject):
    def validateToolbarItem_(self, a):
        return


class TestNSToolbarItem(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSToolbarItemVisibilityPriorityStandard, 0)
        self.assertEqual(AppKit.NSToolbarItemVisibilityPriorityLow, -1000)
        self.assertEqual(AppKit.NSToolbarItemVisibilityPriorityHigh, 1000)
        self.assertEqual(AppKit.NSToolbarItemVisibilityPriorityUser, 2000)

        self.assertIsInstance(AppKit.NSToolbarSeparatorItemIdentifier, str)
        self.assertIsInstance(AppKit.NSToolbarSpaceItemIdentifier, str)
        self.assertIsInstance(AppKit.NSToolbarFlexibleSpaceItemIdentifier, str)

        self.assertIsInstance(AppKit.NSToolbarShowColorsItemIdentifier, str)
        self.assertIsInstance(AppKit.NSToolbarShowFontsItemIdentifier, str)
        self.assertIsInstance(AppKit.NSToolbarCustomizeToolbarItemIdentifier, str)
        self.assertIsInstance(AppKit.NSToolbarPrintItemIdentifier, str)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSToolbarToggleSidebarItemIdentifier, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSToolbarCloudSharingItemIdentifier, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSToolbarItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSToolbarItem.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbarItem.autovalidates)
        self.assertArgIsBOOL(AppKit.NSToolbarItem.setAutovalidates_, 0)
        self.assertResultIsBOOL(AppKit.NSToolbarItem.allowsDuplicatesInToolbar)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(AppKit.NSToolbarItem.isBordered)
        self.assertArgIsBOOL(AppKit.NSToolbarItem.setBordered_, 0)

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(AppKit.NSToolbarItem.isNavigational)
        self.assertArgIsBOOL(AppKit.NSToolbarItem.setNavigational_, 0)

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSToolbarItemHelper.validateToolbarItem_)

    @min_sdk_level("10.12")
    def testProtocolObject(self):
        objc.protocolNamed("NSCloudSharingValidation")

    @min_sdk_level("10.14")
    def testProtocolObject10_14(self):
        objc.protocolNamed("NSToolbarItemValidation")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestNSPointerFunctions(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSPointerFunctionsStrongMemory, (0 << 0))
        self.assertEqual(Foundation.NSPointerFunctionsZeroingWeakMemory, (1 << 0))
        self.assertEqual(Foundation.NSPointerFunctionsOpaqueMemory, (2 << 0))
        self.assertEqual(Foundation.NSPointerFunctionsMallocMemory, (3 << 0))
        self.assertEqual(Foundation.NSPointerFunctionsMachVirtualMemory, (4 << 0))
        self.assertEqual(Foundation.NSPointerFunctionsObjectPersonality, (0 << 8))
        self.assertEqual(Foundation.NSPointerFunctionsOpaquePersonality, (1 << 8))
        self.assertEqual(
            Foundation.NSPointerFunctionsObjectPointerPersonality, (2 << 8)
        )
        self.assertEqual(Foundation.NSPointerFunctionsCStringPersonality, (3 << 8))
        self.assertEqual(Foundation.NSPointerFunctionsStructPersonality, (4 << 8))
        self.assertEqual(Foundation.NSPointerFunctionsIntegerPersonality, (5 << 8))

        self.assertEqual(Foundation.NSPointerFunctionsCopyIn, (1 << 16))

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSPointerFunctionsWeakMemory, 5 << 0)

    def testPropType(self):
        o = Foundation.NSPointerFunctions.alloc().initWithOptions_(0)

        v = o.usesStrongWriteBarrier()
        self.assertTrue((v is True) or (v is False))

        self.assertArgIsBOOL(o.setUsesStrongWriteBarrier_, 0)
        self.assertArgIsBOOL(o.setUsesWeakReadAndWriteBarriers_, 0)

        v = o.usesWeakReadAndWriteBarriers()
        self.assertTrue((v is True) or (v is False))

    @expectedFailure
    def testCallbacks(self):
        self.fail("pointer personality functions")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level


class TestNSTouchBarItem(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTouchBarItem.isVisible)
        # self.assertArgIsBOOL(AppKit.NSTouchBarItem.setVisible_, 0)

    @min_sdk_level("10.12")
    def testConstants(self):
        self.assertEqual(AppKit.NSTouchBarItemPriorityHigh, 1000.0)
        self.assertEqual(AppKit.NSTouchBarItemPriorityNormal, 0.0)
        self.assertEqual(AppKit.NSTouchBarItemPriorityLow, -1000.0)

        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierFixedSpaceSmall, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierFixedSpaceLarge, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierFlexibleSpace, str)
        self.assertIsInstance(AppKit.NSTouchBarItemIdentifierOtherItemsProxy, str)



import objc
from Foundation import NSObject
from InfoWindowController import InfoWindowController


class ToDoAppDelegate(NSObject):
    @objc.IBAction
    def showInfo_(self, sender):
        InfoWindowController.sharedInfoWindowController().showWindow_(sender)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSDistributedLock(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSDistributedLock.tryLock)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFontDescriptor(TestCase):
    def testConvenience(self):
        v = AppKit.NSFontDescriptor.fontDescriptorWithName_size_("Courier", 12)
        d = v[AppKit.NSFontNameAttribute]
        self.assertEqual(v.get(AppKit.NSFontNameAttribute), d)
        self.assertEqual(v.get("no-such-name"), None)
        with self.assertRaises(KeyError):
            v["no-such-name"]

    def testConstants(self):
        self.assertEqual(AppKit.NSFontUnknownClass, (0 << 28))
        self.assertEqual(AppKit.NSFontOldStyleSerifsClass, (1 << 28))
        self.assertEqual(AppKit.NSFontTransitionalSerifsClass, (2 << 28))
        self.assertEqual(AppKit.NSFontModernSerifsClass, (3 << 28))
        self.assertEqual(AppKit.NSFontClarendonSerifsClass, (4 << 28))
        self.assertEqual(AppKit.NSFontSlabSerifsClass, (5 << 28))
        self.assertEqual(AppKit.NSFontFreeformSerifsClass, (7 << 28))
        self.assertEqual(AppKit.NSFontSansSerifClass, (8 << 28))
        self.assertEqual(AppKit.NSFontOrnamentalsClass, (9 << 28))
        self.assertEqual(AppKit.NSFontScriptsClass, (10 << 28))
        self.assertEqual(AppKit.NSFontSymbolicClass, (12 << 28))

        self.assertEqual(AppKit.NSFontFamilyClassMask, (0xF0000000))

        self.assertEqual(AppKit.NSFontItalicTrait, (1 << 0))
        self.assertEqual(AppKit.NSFontBoldTrait, (1 << 1))
        self.assertEqual(AppKit.NSFontExpandedTrait, (1 << 5))
        self.assertEqual(AppKit.NSFontCondensedTrait, (1 << 6))
        self.assertEqual(AppKit.NSFontMonoSpaceTrait, (1 << 10))
        self.assertEqual(AppKit.NSFontVerticalTrait, (1 << 11))
        self.assertEqual(AppKit.NSFontUIOptimizedTrait, (1 << 12))

        self.assertIsInstance(AppKit.NSFontFamilyAttribute, str)
        self.assertIsInstance(AppKit.NSFontNameAttribute, str)
        self.assertIsInstance(AppKit.NSFontFaceAttribute, str)
        self.assertIsInstance(AppKit.NSFontSizeAttribute, str)
        self.assertIsInstance(AppKit.NSFontVisibleNameAttribute, str)
        self.assertIsInstance(AppKit.NSFontMatrixAttribute, str)
        self.assertIsInstance(AppKit.NSFontVariationAttribute, str)
        self.assertIsInstance(AppKit.NSFontCharacterSetAttribute, str)
        self.assertIsInstance(AppKit.NSFontCascadeListAttribute, str)
        self.assertIsInstance(AppKit.NSFontTraitsAttribute, str)
        self.assertIsInstance(AppKit.NSFontFixedAdvanceAttribute, str)
        self.assertIsInstance(AppKit.NSFontFeatureSettingsAttribute, str)
        self.assertIsInstance(AppKit.NSFontColorAttribute, str)
        self.assertIsInstance(AppKit.NSFontSymbolicTrait, str)
        self.assertIsInstance(AppKit.NSFontWeightTrait, str)
        self.assertIsInstance(AppKit.NSFontWidthTrait, str)
        self.assertIsInstance(AppKit.NSFontSlantTrait, str)
        self.assertIsInstance(AppKit.NSFontVariationAxisIdentifierKey, str)
        self.assertIsInstance(AppKit.NSFontVariationAxisMinimumValueKey, str)
        self.assertIsInstance(AppKit.NSFontVariationAxisMaximumValueKey, str)
        self.assertIsInstance(AppKit.NSFontVariationAxisDefaultValueKey, str)
        self.assertIsInstance(AppKit.NSFontVariationAxisNameKey, str)
        self.assertIsInstance(AppKit.NSFontFeatureTypeIdentifierKey, str)
        self.assertIsInstance(AppKit.NSFontFeatureSelectorIdentifierKey, str)

        self.assertEqual(AppKit.NSFontDescriptorTraitItalic, 1 << 0)
        self.assertEqual(AppKit.NSFontDescriptorTraitBold, 1 << 1)
        self.assertEqual(AppKit.NSFontDescriptorTraitExpanded, 1 << 5)
        self.assertEqual(AppKit.NSFontDescriptorTraitCondensed, 1 << 6)
        self.assertEqual(AppKit.NSFontDescriptorTraitMonoSpace, 1 << 10)
        self.assertEqual(AppKit.NSFontDescriptorTraitVertical, 1 << 11)
        self.assertEqual(AppKit.NSFontDescriptorTraitUIOptimized, 1 << 12)
        self.assertEqual(AppKit.NSFontDescriptorTraitTightLeading, 1 << 15)
        self.assertEqual(AppKit.NSFontDescriptorTraitLooseLeading, 1 << 16)
        self.assertEqual(AppKit.NSFontDescriptorClassMask, 0xF0000000)

        self.assertEqual(AppKit.NSFontDescriptorClassUnknown, 0 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassOldStyleSerifs, 1 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassTransitionalSerifs, 2 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassModernSerifs, 3 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassClarendonSerifs, 4 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassSlabSerifs, 5 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassFreeformSerifs, 7 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassSansSerif, 8 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassOrnamentals, 9 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassScripts, 10 << 28)
        self.assertEqual(AppKit.NSFontDescriptorClassSymbolic, 12 << 28)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSFontWeightUltraLight, float)
        self.assertIsInstance(AppKit.NSFontWeightThin, float)
        self.assertIsInstance(AppKit.NSFontWeightLight, float)
        self.assertIsInstance(AppKit.NSFontWeightRegular, float)
        self.assertIsInstance(AppKit.NSFontWeightMedium, float)
        self.assertIsInstance(AppKit.NSFontWeightSemibold, float)
        self.assertIsInstance(AppKit.NSFontWeightBold, float)
        self.assertIsInstance(AppKit.NSFontWeightHeavy, float)
        self.assertIsInstance(AppKit.NSFontWeightBlack, float)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(AppKit.NSFontDescriptorSystemDesignDefault, str)
        self.assertIsInstance(AppKit.NSFontDescriptorSystemDesignSerif, str)
        self.assertIsInstance(AppKit.NSFontDescriptorSystemDesignMonospaced, str)
        self.assertIsInstance(AppKit.NSFontDescriptorSystemDesignRounded, str)

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(AppKit.NSFontTextStyleLargeTitle, str)
        self.assertIsInstance(AppKit.NSFontTextStyleTitle1, str)
        self.assertIsInstance(AppKit.NSFontTextStyleTitle2, str)
        self.assertIsInstance(AppKit.NSFontTextStyleTitle3, str)
        self.assertIsInstance(AppKit.NSFontTextStyleHeadline, str)
        self.assertIsInstance(AppKit.NSFontTextStyleSubheadline, str)
        self.assertIsInstance(AppKit.NSFontTextStyleBody, str)
        self.assertIsInstance(AppKit.NSFontTextStyleCallout, str)
        self.assertIsInstance(AppKit.NSFontTextStyleFootnote, str)
        self.assertIsInstance(AppKit.NSFontTextStyleCaption1, str)
        self.assertIsInstance(AppKit.NSFontTextStyleCaption2, str)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSFontDescriptor.requiresFontAssetRequest)



import Cocoa  # noqa: F401
import objc
from PyObjCTools import AppHelper
from PyObjCTools.TestSupport import TestCase


class TestAppHelper(TestCase):
    def test_callAfter(self):
        pass

    def test_callLater(self):
        pass

    def test_stopEventLoop(self):
        pass

    def test_endSheetMethod(self):
        v = AppHelper.endSheetMethod(lambda x: None)
        self.assertEqual(v.signature, b"v@:@" + objc._C_NSInteger + objc._C_NSInteger)

    def test_runConsoleEventLoop(self):
        pass

    def test_runEventLoop(self):
        pass



import socket
import struct

import CoreFoundation
from PyObjCTools.TestSupport import TestCase, onlyIf

cached_info = None


def onTheNetwork():
    global cached_info
    if cached_info is not None:
        return cached_info

    try:
        socket.gethostbyname("www.apple.com")

        sd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sd.settimeout(1.0)
        try:
            sd.connect(("www.apple.com", 80))
        finally:
            sd.close()

    except OSError:
        cached_info = False
        return False

    cached_info = True
    return True


class TestSocket(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFSocketRef)

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFSocketGetTypeID(), int)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFSocketSuccess, 0)
        self.assertEqual(CoreFoundation.kCFSocketError, -1)
        self.assertEqual(CoreFoundation.kCFSocketTimeout, -2)
        self.assertEqual(CoreFoundation.kCFSocketNoCallBack, 0)
        self.assertEqual(CoreFoundation.kCFSocketReadCallBack, 1)
        self.assertEqual(CoreFoundation.kCFSocketAcceptCallBack, 2)
        self.assertEqual(CoreFoundation.kCFSocketDataCallBack, 3)
        self.assertEqual(CoreFoundation.kCFSocketConnectCallBack, 4)
        self.assertEqual(CoreFoundation.kCFSocketWriteCallBack, 8)
        self.assertEqual(CoreFoundation.kCFSocketAutomaticallyReenableReadCallBack, 1)
        self.assertEqual(CoreFoundation.kCFSocketAutomaticallyReenableAcceptCallBack, 2)
        self.assertEqual(CoreFoundation.kCFSocketAutomaticallyReenableDataCallBack, 3)
        self.assertEqual(CoreFoundation.kCFSocketAutomaticallyReenableWriteCallBack, 8)
        self.assertEqual(CoreFoundation.kCFSocketCloseOnInvalidate, 128)
        self.assertIsInstance(CoreFoundation.kCFSocketCommandKey, str)
        self.assertIsInstance(CoreFoundation.kCFSocketNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFSocketValueKey, str)
        self.assertIsInstance(CoreFoundation.kCFSocketResultKey, str)
        self.assertIsInstance(CoreFoundation.kCFSocketErrorKey, str)
        self.assertIsInstance(CoreFoundation.kCFSocketRegisterCommand, str)
        self.assertIsInstance(CoreFoundation.kCFSocketRetrieveCommand, str)
        self.assertEqual(CoreFoundation.kCFSocketLeaveErrors, 64)

    def testStructs(self):
        o = CoreFoundation.CFSocketSignature()
        self.assertHasAttr(o, "protocolFamily")
        self.assertHasAttr(o, "socketType")
        self.assertHasAttr(o, "protocol")
        self.assertHasAttr(o, "address")

    def testNameRegistry(self):
        p1 = CoreFoundation.CFSocketGetDefaultNameRegistryPortNumber()
        self.assertIsInstance(p1, int)
        CoreFoundation.CFSocketSetDefaultNameRegistryPortNumber(p1 + 1)
        p2 = CoreFoundation.CFSocketGetDefaultNameRegistryPortNumber()
        self.assertIsInstance(p2, int)
        self.assertEqual(p2, p1 + 1)

        CoreFoundation.CFSocketSetDefaultNameRegistryPortNumber(p1)

    @onlyIf(onTheNetwork(), "Test requires a working Internet connection")
    def testSocketFunctions(self):
        data = {}
        state = []

        def callback(sock, kind, address, data, info):
            state.append((sock, kind, address, data, info))

        sock = CoreFoundation.CFSocketCreate(
            None,
            socket.AF_INET,
            socket.SOCK_STREAM,
            0,
            CoreFoundation.kCFSocketReadCallBack
            | CoreFoundation.kCFSocketWriteCallBack,
            callback,
            data,
        )
        self.assertIsInstance(sock, CoreFoundation.CFSocketRef)
        localaddr = struct.pack(">BBHBBBB", 16, socket.AF_INET, 0, 127, 0, 0, 1)
        localaddr += b"\0" * 8

        _ = CoreFoundation.CFSocketCopyAddress(sock)
        err = CoreFoundation.CFSocketSetAddress(sock, localaddr)
        self.assertEqual(err, CoreFoundation.kCFSocketSuccess)

        sd = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        try:
            sock = CoreFoundation.CFSocketCreateWithNative(
                None,
                sd.fileno(),
                CoreFoundation.kCFSocketReadCallBack
                | CoreFoundation.kCFSocketWriteCallBack,
                callback,
                data,
            )
            self.assertIsInstance(sock, CoreFoundation.CFSocketRef)
            n = CoreFoundation.CFSocketGetNative(sock)
            self.assertIsInstance(n, int)
            self.assertEqual(n, sd.fileno())

            ctx = CoreFoundation.CFSocketGetContext(sock, None)
            self.assertIs(ctx, data)
            flags = CoreFoundation.CFSocketGetSocketFlags(sock)
            self.assertIsInstance(flags, int)
            CoreFoundation.CFSocketSetSocketFlags(
                sock,
                CoreFoundation.kCFSocketAutomaticallyReenableReadCallBack
                | CoreFoundation.kCFSocketAutomaticallyReenableAcceptCallBack,
            )
            flags2 = CoreFoundation.CFSocketGetSocketFlags(sock)
            self.assertIsInstance(flags2, int)
            self.assertEqual(
                flags2,
                CoreFoundation.kCFSocketAutomaticallyReenableReadCallBack
                | CoreFoundation.kCFSocketAutomaticallyReenableAcceptCallBack,
            )

            # Note: I don't expect anyone to actually use this api, building
            # struct sockaddr buffers by hand is madness in python.
            ip = socket.gethostbyname("www.apple.com")
            ip = map(int, ip.split("."))

            sockaddr = struct.pack(">BBHBBBB", 16, socket.AF_INET, 80, *ip)
            sockaddr += b"\0" * 8

            e = CoreFoundation.CFSocketConnectToAddress(sock, sockaddr, 1.0)
            self.assertIsInstance(e, int)
            self.assertEqual(e, CoreFoundation.kCFSocketSuccess)

            self.assertResultIsCFRetained(CoreFoundation.CFSocketCopyPeerAddress)
            addr = CoreFoundation.CFSocketCopyPeerAddress(sock)
            self.assertIsInstance(addr, CoreFoundation.CFDataRef)
            self.assertResultIsCFRetained(CoreFoundation.CFSocketCopyAddress)
            addr = CoreFoundation.CFSocketCopyAddress(sock)
            self.assertIsInstance(addr, CoreFoundation.CFDataRef)
            CoreFoundation.CFSocketDisableCallBacks(
                sock,
                CoreFoundation.kCFSocketReadCallBack
                | CoreFoundation.kCFSocketAcceptCallBack,
            )
            CoreFoundation.CFSocketEnableCallBacks(
                sock,
                CoreFoundation.kCFSocketReadCallBack
                | CoreFoundation.kCFSocketAcceptCallBack,
            )

            err = CoreFoundation.CFSocketSendData(sock, None, b"GET / HTTP/1.0", 1.0)
            self.assertEqual(err, CoreFoundation.kCFSocketSuccess)

            ok = CoreFoundation.CFSocketIsValid(sock)
            self.assertIs(ok, True)
            CoreFoundation.CFSocketInvalidate(sock)
            self.assertResultIsBOOL(CoreFoundation.CFSocketIsValid)
            ok = CoreFoundation.CFSocketIsValid(sock)
            self.assertIs(ok, False)
            localaddr = struct.pack(">BBHBBBB", 16, socket.AF_INET, 0, 127, 0, 0, 1)
            localaddr += b"\0" * 8
            signature = CoreFoundation.CFSocketSignature(
                socket.AF_INET, socket.SOCK_STREAM, 0, localaddr
            )

            sock = CoreFoundation.CFSocketCreateWithSocketSignature(
                None,
                signature,
                CoreFoundation.kCFSocketReadCallBack
                | CoreFoundation.kCFSocketWriteCallBack,
                callback,
                data,
            )
            self.assertIsInstance(sock, CoreFoundation.CFSocketRef)
            signature = CoreFoundation.CFSocketSignature(
                socket.AF_INET, socket.SOCK_STREAM, 0, sockaddr
            )
            sock = CoreFoundation.CFSocketCreateConnectedToSocketSignature(
                None,
                signature,
                CoreFoundation.kCFSocketReadCallBack
                | CoreFoundation.kCFSocketWriteCallBack,
                callback,
                data,
                1.0,
            )
            self.assertIsInstance(sock, CoreFoundation.CFSocketRef)
            self.assertResultIsCFRetained(CoreFoundation.CFSocketCreateRunLoopSource)
            src = CoreFoundation.CFSocketCreateRunLoopSource(None, sock, 0)
            self.assertIsInstance(src, CoreFoundation.CFRunLoopSourceRef)

        finally:
            sd.close()

    def testSocketNameServer(self):
        # The documentation says:
        #   Name server functionality is currently inoperable in macOS.
        #
        # Therefore these functions are not available from Python
        self.assertNotHasAttr(CoreFoundation, "CFSocketCopyRegisteredSocketSignature")
        self.assertNotHasAttr(CoreFoundation, "CFSocketCopyRegisteredValue")
        self.assertNotHasAttr(CoreFoundation, "CFSocketRegisterSocketSignature")
        self.assertNotHasAttr(CoreFoundation, "CFSocketRegisterValue")
        self.assertNotHasAttr(CoreFoundation, "CFSocketUnregister")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, max_os_level


class TestNSPrintInfo(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSPortraitOrientation, 0)
        self.assertEqual(AppKit.NSLandscapeOrientation, 1)

        self.assertEqual(AppKit.NSPaperOrientationPortrait, 0)
        self.assertEqual(AppKit.NSPaperOrientationLandscape, 1)

        self.assertEqual(AppKit.NSAutoPagination, 0)
        self.assertEqual(AppKit.NSFitPagination, 1)
        self.assertEqual(AppKit.NSClipPagination, 2)

        self.assertIsInstance(AppKit.NSPrintSpoolJob, str)
        self.assertIsInstance(AppKit.NSPrintPreviewJob, str)
        self.assertIsInstance(AppKit.NSPrintSaveJob, str)
        self.assertIsInstance(AppKit.NSPrintCancelJob, str)

        self.assertIsInstance(AppKit.NSPrintPaperName, str)
        self.assertIsInstance(AppKit.NSPrintPaperSize, str)
        self.assertIsInstance(AppKit.NSPrintOrientation, str)
        self.assertIsInstance(AppKit.NSPrintScalingFactor, str)

        self.assertIsInstance(AppKit.NSPrintLeftMargin, str)
        self.assertIsInstance(AppKit.NSPrintRightMargin, str)
        self.assertIsInstance(AppKit.NSPrintTopMargin, str)
        self.assertIsInstance(AppKit.NSPrintBottomMargin, str)
        self.assertIsInstance(AppKit.NSPrintHorizontallyCentered, str)
        self.assertIsInstance(AppKit.NSPrintVerticallyCentered, str)
        self.assertIsInstance(AppKit.NSPrintHorizontalPagination, str)
        self.assertIsInstance(AppKit.NSPrintVerticalPagination, str)

        self.assertIsInstance(AppKit.NSPrintPrinter, str)
        self.assertIsInstance(AppKit.NSPrintCopies, str)
        self.assertIsInstance(AppKit.NSPrintAllPages, str)
        self.assertIsInstance(AppKit.NSPrintFirstPage, str)
        self.assertIsInstance(AppKit.NSPrintLastPage, str)
        self.assertIsInstance(AppKit.NSPrintMustCollate, str)
        self.assertIsInstance(AppKit.NSPrintReversePageOrder, str)
        self.assertIsInstance(AppKit.NSPrintJobDisposition, str)
        self.assertIsInstance(AppKit.NSPrintSavePath, str)
        self.assertIsInstance(AppKit.NSPrintPagesAcross, str)
        self.assertIsInstance(AppKit.NSPrintPagesDown, str)
        self.assertIsInstance(AppKit.NSPrintTime, str)
        self.assertIsInstance(AppKit.NSPrintDetailedErrorReporting, str)
        self.assertIsInstance(AppKit.NSPrintFaxNumber, str)
        self.assertIsInstance(AppKit.NSPrintPrinterName, str)

        self.assertIsInstance(AppKit.NSPrintHeaderAndFooter, str)

        self.assertIsInstance(AppKit.NSPrintFormName, str)
        self.assertIsInstance(AppKit.NSPrintJobFeatures, str)
        self.assertIsInstance(AppKit.NSPrintManualFeed, str)
        self.assertIsInstance(AppKit.NSPrintPagesPerSheet, str)
        self.assertIsInstance(AppKit.NSPrintPaperFeed, str)

        self.assertEqual(AppKit.NSPrintingPaginationModeAutomatic, 0)
        self.assertEqual(AppKit.NSPrintingPaginationModeFit, 1)
        self.assertEqual(AppKit.NSPrintingPaginationModeClip, 2)

    @max_os_level("10.13")
    def testConstants_not10_14(self):
        self.assertIsInstance(AppKit.NSPrintFaxCoverSheetName, str)
        self.assertIsInstance(AppKit.NSPrintFaxHighResolution, str)
        self.assertIsInstance(AppKit.NSPrintFaxModem, str)
        self.assertIsInstance(AppKit.NSPrintFaxReceiverNames, str)
        self.assertIsInstance(AppKit.NSPrintFaxReceiverNumbers, str)
        self.assertIsInstance(AppKit.NSPrintFaxReturnReceipt, str)
        self.assertIsInstance(AppKit.NSPrintFaxSendTime, str)
        self.assertIsInstance(AppKit.NSPrintFaxTrimPageEnds, str)
        self.assertIsInstance(AppKit.NSPrintFaxUseCoverSheet, str)
        self.assertIsInstance(AppKit.NSPrintFaxJob, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPrintInfo.isHorizontallyCentered)
        self.assertResultIsBOOL(AppKit.NSPrintInfo.isVerticallyCentered)
        self.assertArgIsBOOL(AppKit.NSPrintInfo.setHorizontallyCentered_, 0)
        self.assertArgIsBOOL(AppKit.NSPrintInfo.setVerticallyCentered_, 0)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSPrintSelectionOnly, str)
        self.assertIsInstance(AppKit.NSPrintJobSavingURL, str)
        self.assertIsInstance(AppKit.NSPrintJobSavingFileNameExtensionHidden, str)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSPrintInfo.isSelectionOnly)
        self.assertArgIsBOOL(AppKit.NSPrintInfo.setSelectionOnly_, 0)



import AppKit  # noqa: F401
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSSpellProtocol(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSChangeSpelling")
        objc.protocolNamed("NSIgnoreMisspelledWords")



"""PythonBrowser.py -- a module and/or demo program implementing a Python
object browser.

It can be used in two ways:
1) as a standalone demo app that shows how to use the NSOutlineView class
2) as a module to add an object browser to your app.

For the latter usage, include PythonBrowser.nib in your app bundle,
make sure that PythonBrowser.py and PythonBrowserModel.py can be found
on sys.path, and call

    PythonBrowser.PythonBrowserWindowController(aBrowsableObject)

from your app. The object to be browsed can't be a number, a string or
None, any other kind of object is fine.

To build the demo program, run this line in Terminal.app:

    $ python setup.py py2app -A

This creates a directory "dist" containing PythonBrowser.app. (The
-A option causes the files to be symlinked to the .app bundle instead
of copied. This means you don't have to rebuild the app if you edit the
sources or nibs.)
"""

import sys

import Cocoa
import objc
from PythonBrowserModel import PythonBrowserModel


# class defined in PythonBrowser.nib
class PythonBrowserWindowController(Cocoa.NSWindowController):
    outlineView = objc.IBOutlet()

    def __new__(cls, obj):
        # "Pythonic" constructor
        return cls.alloc().initWithObject_(obj)

    def initWithObject_(self, obj):
        self = self.initWithWindowNibName_("PythonBrowser")
        self.setWindowTitleForObject_(obj)
        self.model = PythonBrowserModel.alloc().initWithObject_(obj)
        self.outlineView.setDataSource_(self.model)
        self.outlineView.setDelegate_(self.model)
        self.outlineView.setTarget_(self)
        self.outlineView.setDoubleAction_(b"doubleClick:")
        self.window().makeFirstResponder_(self.outlineView)
        self.showWindow_(self)
        # The window controller doesn't need to be retained (referenced)
        # anywhere, so we pretend to have a reference to ourselves to avoid
        # being garbage collected before the window is closed. The extra
        # reference will be released in self.windowWillClose_()
        self.retain()
        return self

    def windowWillClose_(self, notification):
        # see comment in self.initWithObject_()
        self.autorelease()

    def setWindowTitleForObject_(self, obj):
        if hasattr(obj, "__name__"):
            title = f"PythonBrowser -- {type(obj).__name__}: {obj.__name__}"
        else:
            title = f"PythonBrowser -- {type(obj).__name__}"
        self.window().setTitle_(title)

    def setObject_(self, obj):
        self.setWindowTitleForObject_(obj)
        self.model.setObject_(obj)
        self.outlineView.reloadData()

    @objc.IBAction
    def doubleClick_(self, sender):
        # Open a new browser window for each selected expandable item
        for row in self.outlineView.selectedRowEnumerator():
            item = self.outlineView.itemAtRow_(row)
            if item.isExpandable():
                PythonBrowserWindowController(item.object)

    @objc.IBAction
    def pickRandomModule_(self, sender):
        """Test method, hooked up from the "Pick Random Module" menu in
        MainMenu.nib, to test changing the browsed object after the window
        has been created."""
        from random import choice

        mod = None
        while mod is None:
            mod = sys.modules[choice(sys.modules.keys())]
        self.setObject_(mod)


class PythonBrowserAppDelegate(Cocoa.NSObject):
    def applicationDidFinishLaunching_(self, notification):
        self.newBrowser_(self)

    @objc.IBAction
    def newBrowser_(self, sender):
        # The PythonBrowserWindowController instance will retain itself,
        # so we don't (have to) keep track of all instances here.
        PythonBrowserWindowController(sys)


if __name__ == "__main__":
    from PyObjCTools import AppHelper

    AppHelper.runEventLoop()



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSCalendar(TestCase):
    def testConstants(self):
        self.assertEqual(
            Foundation.NSEraCalendarUnit, CoreFoundation.kCFCalendarUnitEra
        )
        self.assertEqual(
            Foundation.NSYearCalendarUnit, CoreFoundation.kCFCalendarUnitYear
        )
        self.assertEqual(
            Foundation.NSMonthCalendarUnit, CoreFoundation.kCFCalendarUnitMonth
        )
        self.assertEqual(
            Foundation.NSDayCalendarUnit, CoreFoundation.kCFCalendarUnitDay
        )
        self.assertEqual(
            Foundation.NSHourCalendarUnit, CoreFoundation.kCFCalendarUnitHour
        )
        self.assertEqual(
            Foundation.NSMinuteCalendarUnit, CoreFoundation.kCFCalendarUnitMinute
        )
        self.assertEqual(
            Foundation.NSSecondCalendarUnit, CoreFoundation.kCFCalendarUnitSecond
        )
        self.assertEqual(
            Foundation.NSWeekCalendarUnit, CoreFoundation.kCFCalendarUnitWeek
        )
        self.assertEqual(
            Foundation.NSWeekdayCalendarUnit, CoreFoundation.kCFCalendarUnitWeekday
        )
        self.assertEqual(
            Foundation.NSWeekdayOrdinalCalendarUnit,
            CoreFoundation.kCFCalendarUnitWeekdayOrdinal,
        )

        self.assertEqual(
            Foundation.NSWrapCalendarComponents,
            CoreFoundation.kCFCalendarComponentsWrap,
        )

        self.assertEqual(Foundation.NSUndefinedDateComponent, Foundation.NSIntegerMax)
        self.assertEqual(Foundation.NSDateComponentUndefined, Foundation.NSIntegerMax)

        self.assertEqual(
            Foundation.NSCalendarUnitEra, CoreFoundation.kCFCalendarUnitEra
        )
        self.assertEqual(
            Foundation.NSCalendarUnitYear, CoreFoundation.kCFCalendarUnitYear
        )
        self.assertEqual(
            Foundation.NSCalendarUnitMonth, CoreFoundation.kCFCalendarUnitMonth
        )
        self.assertEqual(
            Foundation.NSCalendarUnitDay, CoreFoundation.kCFCalendarUnitDay
        )
        self.assertEqual(
            Foundation.NSCalendarUnitHour, CoreFoundation.kCFCalendarUnitHour
        )
        self.assertEqual(
            Foundation.NSCalendarUnitMinute, CoreFoundation.kCFCalendarUnitMinute
        )
        self.assertEqual(
            Foundation.NSCalendarUnitSecond, CoreFoundation.kCFCalendarUnitSecond
        )
        self.assertEqual(
            Foundation.NSCalendarUnitWeekday, CoreFoundation.kCFCalendarUnitWeekday
        )
        self.assertEqual(
            Foundation.NSCalendarUnitWeekdayOrdinal,
            CoreFoundation.kCFCalendarUnitWeekdayOrdinal,
        )
        self.assertEqual(
            Foundation.NSCalendarUnitQuarter, CoreFoundation.kCFCalendarUnitQuarter
        )
        self.assertEqual(
            Foundation.NSCalendarUnitWeekOfMonth,
            CoreFoundation.kCFCalendarUnitWeekOfMonth,
        )
        self.assertEqual(
            Foundation.NSCalendarUnitWeekOfYear,
            CoreFoundation.kCFCalendarUnitWeekOfYear,
        )
        self.assertEqual(
            Foundation.NSCalendarUnitYearForWeekOfYear,
            CoreFoundation.kCFCalendarUnitYearForWeekOfYear,
        )
        self.assertEqual(Foundation.NSCalendarUnitNanosecond, (1 << 15))
        self.assertEqual(Foundation.NSCalendarUnitCalendar, (1 << 20))
        self.assertEqual(Foundation.NSCalendarUnitTimeZone, (1 << 21))
        self.assertEqual(Foundation.NSCalendarWrapComponents, (1 << 0))
        self.assertEqual(Foundation.NSCalendarMatchStrictly, (1 << 1))
        self.assertEqual(Foundation.NSCalendarSearchBackwards, (1 << 2))
        self.assertEqual(
            Foundation.NSCalendarMatchPreviousTimePreservingSmallerUnits, (1 << 8)
        )
        self.assertEqual(
            Foundation.NSCalendarMatchNextTimePreservingSmallerUnits, (1 << 9)
        )
        self.assertEqual(Foundation.NSCalendarMatchNextTime, (1 << 10))
        self.assertEqual(Foundation.NSCalendarMatchFirst, (1 << 12))
        self.assertEqual(Foundation.NSCalendarMatchLast, (1 << 13))

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(
            Foundation.NSQuarterCalendarUnit, CoreFoundation.kCFCalendarUnitQuarter
        )

        self.assertIsInstance(Foundation.NSCalendarIdentifierGregorian, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierBuddhist, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierChinese, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierCoptic, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierEthiopicAmeteMihret, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierEthiopicAmeteAlem, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierHebrew, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierISO8601, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierIndian, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierIslamic, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierIslamicCivil, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierJapanese, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierPersian, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierRepublicOfChina, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(
            Foundation.NSWeekOfMonthCalendarUnit,
            CoreFoundation.kCFCalendarUnitWeekOfMonth,
        )
        self.assertEqual(
            Foundation.NSWeekOfYearCalendarUnit,
            CoreFoundation.kCFCalendarUnitWeekOfYear,
        )
        self.assertEqual(
            Foundation.NSYearForWeekOfYearCalendarUnit,
            CoreFoundation.kCFCalendarUnitYearForWeekOfYear,
        )
        self.assertEqual(Foundation.NSCalendarCalendarUnit, (1 << 20))
        self.assertEqual(Foundation.NSTimeZoneCalendarUnit, (1 << 21))

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(Foundation.NSCalendarDayChangedNotification, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(Foundation.NSCalendarIdentifierIslamicTabular, str)
        self.assertIsInstance(Foundation.NSCalendarIdentifierIslamicUmmAlQura, str)

    @min_os_level("10.5")
    def testMethods10_5(self):
        Foundation.NSCalendar.currentCalendar()

        self.assertResultIsBOOL(
            Foundation.NSCalendar.rangeOfUnit_startDate_interval_forDate_
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.rangeOfUnit_startDate_interval_forDate_, 1
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.rangeOfUnit_startDate_interval_forDate_, 2
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(Foundation.NSDateComponents.isLeapMonth)
        self.assertArgIsBOOL(Foundation.NSDateComponents.setLeapMonth_, 0)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsOut(Foundation.NSCalendar.getEra_year_month_day_fromDate_, 0)
        self.assertArgIsOut(Foundation.NSCalendar.getEra_year_month_day_fromDate_, 1)
        self.assertArgIsOut(Foundation.NSCalendar.getEra_year_month_day_fromDate_, 2)
        self.assertArgIsOut(Foundation.NSCalendar.getEra_year_month_day_fromDate_, 3)

        self.assertArgIsOut(
            Foundation.NSCalendar.getEra_yearForWeekOfYear_weekOfYear_weekday_fromDate_,
            0,
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getEra_yearForWeekOfYear_weekOfYear_weekday_fromDate_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getEra_yearForWeekOfYear_weekOfYear_weekday_fromDate_,
            2,
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getEra_yearForWeekOfYear_weekOfYear_weekday_fromDate_,
            3,
        )

        self.assertArgIsOut(
            Foundation.NSCalendar.getHour_minute_second_nanosecond_fromDate_, 0
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getHour_minute_second_nanosecond_fromDate_, 1
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getHour_minute_second_nanosecond_fromDate_, 2
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.getHour_minute_second_nanosecond_fromDate_, 3
        )

        self.assertResultIsBOOL(
            Foundation.NSCalendar.isDate_equalToDate_toUnitGranularity_
        )
        self.assertResultIsBOOL(Foundation.NSCalendar.isDate_inSameDayAsDate_)
        self.assertResultIsBOOL(Foundation.NSCalendar.isDateInToday_)
        self.assertResultIsBOOL(Foundation.NSCalendar.isDateInTomorrow_)
        self.assertResultIsBOOL(Foundation.NSCalendar.isDateInWeekend_)
        self.assertResultIsBOOL(
            Foundation.NSCalendar.rangeOfWeekendStartDate_interval_containingDate_
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.rangeOfWeekendStartDate_interval_containingDate_, 0
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.rangeOfWeekendStartDate_interval_containingDate_, 1
        )
        self.assertResultIsBOOL(
            Foundation.NSCalendar.nextWeekendStartDate_interval_options_afterDate_
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.nextWeekendStartDate_interval_options_afterDate_, 0
        )
        self.assertArgIsOut(
            Foundation.NSCalendar.nextWeekendStartDate_interval_options_afterDate_, 1
        )
        self.assertArgIsBlock(
            Foundation.NSCalendar.enumerateDatesStartingAfterDate_matchingComponents_options_usingBlock_,  # noqa: B950
            3,
            b"v@Zo^Z",
        )
        self.assertResultIsBOOL(Foundation.NSCalendar.date_matchesComponents_)

        self.assertResultIsBOOL(Foundation.NSDateComponents.isValidDate)
        self.assertResultIsBOOL(Foundation.NSDateComponents.isValidDateInCalendar_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTableRowView(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSTableRowView.isEmphasized)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setEmphasized_, 0)
        self.assertResultIsBOOL(AppKit.NSTableRowView.isGroupRowStyle)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setGroupRowStyle_, 0)
        self.assertResultIsBOOL(AppKit.NSTableRowView.isSelected)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setSelected_, 0)
        self.assertResultIsBOOL(AppKit.NSTableRowView.isFloating)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setFloating_, 0)
        self.assertResultIsBOOL(AppKit.NSTableRowView.isTargetForDropOperation)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setTargetForDropOperation_, 0)
        self.assertResultIsBOOL(AppKit.NSTableRowView.isNextRowSelected)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setNextRowSelected_, 0)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSTableRowView.isPreviousRowSelected)
        self.assertArgIsBOOL(AppKit.NSTableRowView.setPreviousRowSelected_, 0)



import Cocoa
import PyObjCTools.AppCategories  # noqa: F401
from PyObjCTools.TestSupport import TestCase, onlyIf


class TestAppKitCategories(TestCase):
    # Note: These tests don't actually test anything beyond asserting that
    # the code doesn't crash...

    def testSavedGraphicsState(self):
        with Cocoa.NSGraphicsContext.savedGraphicsState():
            pass

    @onlyIf(
        hasattr(Cocoa, "NSAnimationContext"), "Test for NSAnimationContext category"
    )
    def testAnimationContext(self):
        with Cocoa.NSAnimationContext:
            pass



"""
Script for building the example, alternative to the Xcode project.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

setup(
    name="MutableTableModel",
    app=["main.py"],
    data_files=["English.lproj"],
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase

# Foundation.NSZonePtr = getattr(Foundation, 'NSZone*')


class TestNSZone(TestCase):
    def testWithZones(self):
        obj = Foundation.NSObject.allocWithZone_(None).init()
        zone = obj.zone()
        self.assertIsNot(zone, None)
        self.assertNotEqual(zone.__pointer__, 0)

        obj2 = Foundation.NSObject.allocWithZone_(zone).init()
        zone2 = obj2.zone()
        self.assertEqual(zone.__pointer__, zone2.__pointer__)

        self.assertRaises(TypeError, Foundation.NSObject.allocWithZone_, 10)
        # self.assertRaises(TypeError, Foundation.NSObject.allocWithZone_, objc.NULL)

    def testNoMallocAndFriends(self):
        import Foundation

        self.assertNotHasAttr(Foundation, "NSZoneMalloc")
        self.assertNotHasAttr(Foundation, "NSZoneCalloc")
        self.assertNotHasAttr(Foundation, "NSZoneRealloc")
        self.assertNotHasAttr(Foundation, "NSZoneFree")
        self.assertNotHasAttr(Foundation, "NSZoneFromPointer")
        self.assertNotHasAttr(Foundation, "NSAllocateCollectable")
        self.assertNotHasAttr(Foundation, "NSReallocateCollectable")
        self.assertNotHasAttr(Foundation, "NSAllocateMemoryPages")
        self.assertNotHasAttr(Foundation, "NSDeallocateMemoryPages")
        self.assertNotHasAttr(Foundation, "NSCopyMemoryPages")

    def testConstants(self):
        self.assertEqual(Foundation.NSScannedOption, (1 << 0))
        self.assertEqual(Foundation.NSCollectorDisabledOption, (1 << 1))

    def testMakeCollectable(self):
        v = Foundation.NSMakeCollectable

        o = Foundation.NSObject.alloc().init()
        CoreFoundation.CFRetain(o)
        v = Foundation.NSMakeCollectable(o)
        self.assertIs(v, o)
        v = Foundation.NSMakeCollectable(None)
        self.assertIs(v, None)

    def testInfoFunctions(self):
        v = Foundation.NSPageSize()
        self.assertIsInstance(v, int)
        v = Foundation.NSLogPageSize()
        self.assertIsInstance(v, int)
        v = Foundation.NSRoundUpToMultipleOfPageSize(500)
        self.assertIsInstance(v, int)
        v = Foundation.NSRoundDownToMultipleOfPageSize(500)
        self.assertIsInstance(v, int)
        v = Foundation.NSRealMemoryAvailable()
        self.assertIsInstance(v, int)

    def testZoneCreation(self):
        z = Foundation.NSDefaultMallocZone()
        if z is not None:
            self.assertIsInstance(z, Foundation.NSZonePtr)
        z = Foundation.NSCreateZone(5000, 100, True)
        self.assertIsInstance(z, Foundation.NSZonePtr)
        Foundation.NSSetZoneName(z, "Hello World")
        nm = Foundation.NSZoneName(z)
        self.assertEqual(nm, "Hello World")

        Foundation.NSRecycleZone(z)
        z = None



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPDFInfo(TestCase):
    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPDFInfo.isFileExtensionHidden)
        self.assertArgIsBOOL(AppKit.NSPDFInfo.setFileExtensionHidden_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSColorSpace(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSUnknownColorSpaceModel, -1)
        self.assertEqual(AppKit.NSGrayColorSpaceModel, 0)
        self.assertEqual(AppKit.NSRGBColorSpaceModel, 1)
        self.assertEqual(AppKit.NSCMYKColorSpaceModel, 2)
        self.assertEqual(AppKit.NSLABColorSpaceModel, 3)
        self.assertEqual(AppKit.NSDeviceNColorSpaceModel, 4)
        self.assertEqual(AppKit.NSIndexedColorSpaceModel, 5)
        self.assertEqual(AppKit.NSPatternColorSpaceModel, 6)

        self.assertEqual(AppKit.NSColorSpaceModelUnknown, -1)
        self.assertEqual(AppKit.NSColorSpaceModelGray, 0)
        self.assertEqual(AppKit.NSColorSpaceModelRGB, 1)
        self.assertEqual(AppKit.NSColorSpaceModelCMYK, 2)
        self.assertEqual(AppKit.NSColorSpaceModelLAB, 3)
        self.assertEqual(AppKit.NSColorSpaceModelDeviceN, 4)
        self.assertEqual(AppKit.NSColorSpaceModelIndexed, 5)
        self.assertEqual(AppKit.NSColorSpaceModelPatterned, 6)

    def testMethods(self):
        self.assertArgHasType(
            AppKit.NSColorSpace.initWithColorSyncProfile_, 0, b"^{OpaqueCMProfileRef=}"
        )
        self.assertResultHasType(
            AppKit.NSColorSpace.colorSyncProfile, b"^{OpaqueCMProfileRef=}"
        )



import time

import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestTimeZone(TestCase):
    def testTypes(self):
        try:
            if objc.lookUpClass("NSTimeZone") is CoreFoundation.CFTimeZoneRef:
                return
        except objc.error:
            pass
        self.assertIsCFType(CoreFoundation.CFTimeZoneRef)

    def testTypeID(self):
        value = CoreFoundation.CFTimeZoneGetTypeID()
        self.assertIsInstance(value, int)

    def testSystemZone(self):
        zone = CoreFoundation.CFTimeZoneCopySystem()
        self.assertIsInstance(zone, CoreFoundation.CFTimeZoneRef)

        CoreFoundation.CFTimeZoneSetDefault

    def testResetSystem(self):
        v = CoreFoundation.CFTimeZoneResetSystem()
        self.assertIs(v, None)

    def testCopyDefault(self):
        zone = CoreFoundation.CFTimeZoneCopyDefault()
        self.assertIsInstance(zone, CoreFoundation.CFTimeZoneRef)

    def testNames(self):
        self.assertResultIsCFRetained(CoreFoundation.CFTimeZoneCopyKnownNames)
        array = CoreFoundation.CFTimeZoneCopyKnownNames()
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        self.assertNotEqual(len(array), 0)
        for nm in array:
            self.assertIsInstance(nm, str)

    def testAbbreviationDict(self):
        abbrevs = CoreFoundation.CFTimeZoneCopyAbbreviationDictionary()
        self.assertIsInstance(abbrevs, CoreFoundation.CFDictionaryRef)
        for key, value in abbrevs.items():
            self.assertIsInstance(key, str)
            self.assertIsInstance(value, str)

    @min_os_level("10.6")
    def testAbbrievationDictSetting(self):
        # Setting the dictionary is technically also possible
        # on 10.5, but the code below causes a crash, even when
        # rewritten als plan Objective-C.
        abbrevs = CoreFoundation.CFTimeZoneCopyAbbreviationDictionary()
        newmap = abbrevs.mutableCopy()
        newmap["AAA"] = "Europe/Amsterdam"

        v = CoreFoundation.CFTimeZoneSetAbbreviationDictionary(newmap)
        self.assertIs(v, None)
        try:
            map2 = CoreFoundation.CFTimeZoneCopyAbbreviationDictionary()
            self.assertIsInstance(map2, CoreFoundation.CFDictionaryRef)
            self.assertEqual(map2["AAA"], "Europe/Amsterdam")
        finally:
            CoreFoundation.CFTimeZoneSetAbbreviationDictionary(abbrevs)

    def testZoneObject(self):
        with open("/usr/share/zoneinfo/posixrules", "rb") as fp:
            data = fp.read()
        zone = CoreFoundation.CFTimeZoneCreate(None, "Europe/Amsterdam", data)
        self.assertIsInstance(zone, CoreFoundation.CFTimeZoneRef)
        self.assertResultIsCFRetained(
            CoreFoundation.CFTimeZoneCreateWithTimeIntervalFromGMT
        )
        zone = CoreFoundation.CFTimeZoneCreateWithTimeIntervalFromGMT(None, 3600)
        self.assertIsInstance(zone, CoreFoundation.CFTimeZoneRef)
        offset = CoreFoundation.CFTimeZoneGetSecondsFromGMT(zone, time.time())
        self.assertEqual(offset, 3600)

        zone = CoreFoundation.CFTimeZoneCreateWithName(None, "Europe/Amsterdam", True)
        self.assertIsInstance(zone, CoreFoundation.CFTimeZoneRef)
        name = CoreFoundation.CFTimeZoneGetName(zone)
        self.assertEqual(name, "Europe/Amsterdam")

        data = CoreFoundation.CFTimeZoneGetData(zone)
        self.assertIsInstance(data, CoreFoundation.CFDataRef)
        abbrev = CoreFoundation.CFTimeZoneCopyAbbreviation(zone, time.time())
        self.assertIsInstance(abbrev, str)
        dt = CoreFoundation.CFGregorianDate(
            year=2008, month=7, day=1, hour=12, minute=0, second=0
        )

        r = CoreFoundation.CFTimeZoneIsDaylightSavingTime(
            zone, CoreFoundation.CFGregorianDateGetAbsoluteTime(dt, zone)
        )
        self.assertIs(r, True)
        dt = CoreFoundation.CFGregorianDate(
            year=2008, month=11, day=1, hour=12, minute=0, second=0
        )

        r = CoreFoundation.CFTimeZoneIsDaylightSavingTime(
            zone, CoreFoundation.CFGregorianDateGetAbsoluteTime(dt, zone)
        )
        self.assertIn(r, (False, True))
        offset = CoreFoundation.CFTimeZoneGetDaylightSavingTimeOffset(
            zone, CoreFoundation.CFGregorianDateGetAbsoluteTime(dt, zone)
        )
        self.assertIsInstance(offset, float)
        dt = CoreFoundation.CFTimeZoneGetNextDaylightSavingTimeTransition(
            zone, CoreFoundation.CFGregorianDateGetAbsoluteTime(dt, zone)
        )
        self.assertIsInstance(dt, float)
        nm = CoreFoundation.CFTimeZoneCopyLocalizedName(
            zone,
            CoreFoundation.kCFTimeZoneNameStyleShortStandard,
            CoreFoundation.CFLocaleCopyCurrent(),
        )
        self.assertIsInstance(nm, str)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleStandard, 0)
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleShortStandard, 1)
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleDaylightSaving, 2)
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleShortDaylightSaving, 3)
        self.assertIsInstance(
            CoreFoundation.kCFTimeZoneSystemTimeZoneDidChangeNotification, str
        )
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleGeneric, 4)
        self.assertEqual(CoreFoundation.kCFTimeZoneNameStyleShortGeneric, 5)



import CoreFoundation
from PyObjCTools.TestSupport import TestCase
import objc


class TestNotificationCenter(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFNotificationCenterRef)

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFNotificationCenterGetTypeID(), int)

    def testGetting(self):
        ref = CoreFoundation.CFNotificationCenterGetLocalCenter()
        self.assertIsInstance(ref, CoreFoundation.CFNotificationCenterRef)
        ref = CoreFoundation.CFNotificationCenterGetDistributedCenter()
        self.assertIsInstance(ref, CoreFoundation.CFNotificationCenterRef)
        ref = CoreFoundation.CFNotificationCenterGetDarwinNotifyCenter()
        self.assertIsInstance(ref, CoreFoundation.CFNotificationCenterRef)

    def testSending(self):
        ref = CoreFoundation.CFNotificationCenterGetLocalCenter()
        self.assertIsInstance(ref, CoreFoundation.CFNotificationCenterRef)
        notifications = []

        @objc.callbackFor(CoreFoundation.CFNotificationCenterAddObserver)
        def observe(center, observer, name, value, userInfo):
            notifications.append((center, observer, name, value, userInfo))

        self.assertArgHasType(CoreFoundation.CFNotificationCenterAddObserver, 1, b"@")
        self.assertArgIsFunction(
            CoreFoundation.CFNotificationCenterAddObserver, 2, b"v@@@@@", True
        )
        self.assertArgHasType(CoreFoundation.CFNotificationCenterAddObserver, 4, b"@")

        args = {}
        args["object"] = "object"
        args["pyobjc.test"] = "pyobjc.test"

        CoreFoundation.CFNotificationCenterAddObserver(
            ref,
            args["object"],
            observe,
            args["pyobjc.test"],
            ref,
            CoreFoundation.CFNotificationSuspensionBehaviorDeliverImmediately,
        )

        CoreFoundation.CFNotificationCenterPostNotificationWithOptions(
            ref,
            "pyobjc.test",
            ref,
            {"name": "value"},
            CoreFoundation.kCFNotificationPostToAllSessions,
        )
        self.assertEqual(len(notifications), 1)
        info = notifications[-1]
        self.assertIs(info[0], ref)
        self.assertEqual(info[1], "object")
        self.assertEqual(info[2], "pyobjc.test")
        self.assertIs(info[3], ref)
        self.assertEqual(info[4], {"name": "value"})
        CoreFoundation.CFNotificationCenterPostNotification(
            ref, "pyobjc.test", ref, {"name2": "value2"}, True
        )
        self.assertEqual(len(notifications), 2)
        info = notifications[-1]
        self.assertIs(info[0], ref)
        self.assertEqual(info[1], "object")
        self.assertEqual(info[2], "pyobjc.test")
        self.assertIs(info[3], ref)
        self.assertEqual(info[4], {"name2": "value2"})
        self.assertArgHasType(
            CoreFoundation.CFNotificationCenterRemoveObserver, 1, b"@"
        )
        self.assertArgHasType(
            CoreFoundation.CFNotificationCenterRemoveObserver, 3, b"@"
        )
        CoreFoundation.CFNotificationCenterRemoveObserver(
            ref, args["object"], args["pyobjc.test"], ref
        )

        self.assertArgHasType(
            CoreFoundation.CFNotificationCenterPostNotificationWithOptions, 2, b"@"
        )
        CoreFoundation.CFNotificationCenterPostNotificationWithOptions(
            ref,
            "pyobjc.test",
            ref,
            {"name": "value"},
            CoreFoundation.kCFNotificationPostToAllSessions,
        )
        self.assertEqual(len(notifications), 2)

        CoreFoundation.CFNotificationCenterAddObserver(
            ref,
            args["object"],
            observe,
            args["pyobjc.test"],
            ref,
            CoreFoundation.CFNotificationSuspensionBehaviorDeliverImmediately,
        )

        self.assertArgHasType(
            CoreFoundation.CFNotificationCenterPostNotification, 2, b"@"
        )
        self.assertArgIsBOOL(CoreFoundation.CFNotificationCenterPostNotification, 4)
        CoreFoundation.CFNotificationCenterPostNotification(
            ref, "pyobjc.test", ref, {"name2": "value2"}, True
        )
        self.assertEqual(len(notifications), 3)
        CoreFoundation.CFNotificationCenterRemoveEveryObserver(ref, args["object"])
        CoreFoundation.CFNotificationCenterPostNotification(
            ref, "pyobjc.test", ref, {"name2": "value2"}, True
        )
        self.assertEqual(len(notifications), 3)

    def testConstants(self):
        self.assertEqual(CoreFoundation.CFNotificationSuspensionBehaviorDrop, 1)
        self.assertEqual(CoreFoundation.CFNotificationSuspensionBehaviorCoalesce, 2)
        self.assertEqual(CoreFoundation.CFNotificationSuspensionBehaviorHold, 3)
        self.assertEqual(
            CoreFoundation.CFNotificationSuspensionBehaviorDeliverImmediately, 4
        )

        self.assertEqual(CoreFoundation.kCFNotificationDeliverImmediately, 1)
        self.assertEqual(CoreFoundation.kCFNotificationPostToAllSessions, 2)
